// **********************************************
// Command Handler
// Instant WEB Application: www.progamma.com
// Project : PRJ Handling Debug
// **********************************************
using System;
using System.Reflection;
using com.progamma;
using com.progamma.ids;

// **********************************************
// Command Handler
// **********************************************
[Serializable]
public sealed class MyCmdHandler : CmdHandler
{
  static int CMDS_OFFSET = 0;
  static int CMD_OFFSET = 0;

  // **********************************************
  // Costruttore
  // **********************************************
  public MyCmdHandler(WebEntryPoint p)
    : base(p)
  {
    //
    CmdSets[MyGlb.CMDS_FORMSLIST] = new ACommand();
    CmdSets[MyGlb.CMDS_FORMSLIST].iGuid = "06F2D873-573B-442A-A1B2-D2B1B5338E42";
    CmdSets[MyGlb.CMDS_FORMSLIST].Index = MyGlb.CMDS_FORMSLIST;
    CmdSets[MyGlb.CMDS_FORMSLIST].InitStatus(true, true, false);
    CmdSets[MyGlb.CMDS_FORMSLIST].SetNumCommands(1);
    CmdSets[MyGlb.CMDS_FORMSLIST].Caption = "Forms list";
    CmdSets[MyGlb.CMDS_FORMSLIST].ToolTip = "Which commands will be included here?";
    CmdSets[MyGlb.CMDS_FORMSLIST].Level = 1;
    CmdSets[MyGlb.CMDS_FORMSLIST].IsMenu = true;
    CmdSets[MyGlb.CMDS_FORMSLIST].IsToolbar = false;
    CmdSets[MyGlb.CMDS_FORMSLIST].ShowNames = false;
    CmdSets[MyGlb.CMDS_FORMSLIST].Code = "FORMSLIST";
    CmdLin[MyGlb.CMD_PROCEDURE1] = new ACommand();
    CmdSets[MyGlb.CMDS_FORMSLIST].AddCommand(CmdLin[MyGlb.CMD_PROCEDURE1]);
    CmdLin[MyGlb.CMD_PROCEDURE1].Parent = CmdSets[MyGlb.CMDS_FORMSLIST];
    CmdLin[MyGlb.CMD_PROCEDURE1].iGuid = "AB2B3197-72EA-40BF-A4EC-90EEACBC302A";
    CmdLin[MyGlb.CMD_PROCEDURE1].Caption = "Procedure 1";
    CmdLin[MyGlb.CMD_PROCEDURE1].ToolTip = "";
    CmdLin[MyGlb.CMD_PROCEDURE1].RequireConfirmation = false;
    CmdLin[MyGlb.CMD_PROCEDURE1].IsMenu = true;
    CmdLin[MyGlb.CMD_PROCEDURE1].IsToolbar = false;
    CmdLin[MyGlb.CMD_PROCEDURE1].ShowNames = false;
    CmdLin[MyGlb.CMD_PROCEDURE1].ToolName = "Procedure";
    CmdLin[MyGlb.CMD_PROCEDURE1].UseHilight = false;
    CmdLin[MyGlb.CMD_PROCEDURE1].Code = "PROCEDURE1";
    CmdLin[MyGlb.CMD_PROCEDURE1].Index = MyGlb.CMD_PROCEDURE1;
    CmdLin[MyGlb.CMD_PROCEDURE1].Level = 2;
    CmdLin[MyGlb.CMD_PROCEDURE1].InitStatus(true, true, false);
    p.MenuType = 1;
    //
    ResetRoles(p);
  }

  // **********************************************
  // Appende gli oggetti al CmdHandler fornito
  // **********************************************
  public override void AppendTo(CmdHandler Dst)
  {
    // Se ho command set
    if (CmdSets.Length > 0)
    {
      // Se  la prima volta, appendo
      if (MyCmdHandler.CMDS_OFFSET == 0)
      {
        // Per cominciare appendo tutti i CmdSet contenuti in Dst dentro al mio array
        int idx = Dst.CmdSets.Length;
        if (idx == 0) idx = 1;    // L'array dei cmdset  1-based
        Array.Resize(ref Dst.CmdSets, idx + CmdSets.Length - 1);
        for (int i = 1; i < CmdSets.Length; i++)
        {
          ACommand c = CmdSets[i];
          if (c == null)
            continue;
          //
          if (c.IdxForm > 0)
            c.IdxForm += MainFrm.CompOwner.iFrmOffs.intValue();   // Shifto le referenze alle form
          //
          c.Index = idx;
          Dst.CmdSets[idx++] = c;
        }
      }
      else // Il componente  gi stato inizializzato in precedenza... infilo i CmdSet dove li ho messi l'ultima volta
      {
        // L'array di mio padre potrebbe essere pi grande del mio se i componenti sono "dinamici" e vengono caricati in un 
        // ordine diverso da quello precedente (es: app->cmp1->cmp2 e poi, in una sessione differente app->cmp2->cmp1)
        Array.Resize(ref Dst.CmdSets, Math.Max(CmdSets.Length, Dst.CmdSets.Length));
        Array.Copy(CmdSets, MyCmdHandler.CMDS_OFFSET + 1, Dst.CmdSets, MyCmdHandler.CMDS_OFFSET + 1, CmdSets.Length - (MyCmdHandler.CMDS_OFFSET + 1));
      }
      //
      // Se non l'ho ancora fatto shifto tutte le costanti contenute in MyGlb
      if (MyCmdHandler.CMDS_OFFSET == 0)
      {
        // Il mio offset  il numero di commandset di tutti i miei padri...
        // Per calcolarlo guardo a dove  finito il mio primo commandset (l'elemento 0  NULL)
        // Quello  l'offset dei miei commandset rispetto all'array che contiene tutti i commandset di tutti i componenti
        MyCmdHandler.CMDS_OFFSET = CmdSets[1].Index - 1;
        //
        String tn = (MainFrm.CompNameSpace != null ? MainFrm.CompNameSpace + "." : "");
        FieldInfo[] props = Assembly.GetExecutingAssembly().GetType(tn + "MyGlb").GetFields();
        for (int j = 0; j < props.Length; j++)
        {
          FieldInfo p = props[j];
          if (p.Name.StartsWith("CMDS_") || p.Name.Equals("MAX_COMMAND_SETS"))
            p.SetValue(null, ((int)p.GetValue(null)) + MyCmdHandler.CMDS_OFFSET);
        }
      }
    }
    //
    // Se ho comandi
    if (CmdLin.Length > 0)
    {
      // Poi proseguo con i comandi
      // Se  la prima volta, appendo
      if (MyCmdHandler.CMD_OFFSET == 0)
      {
        int idx = Dst.CmdLin.Length;
        if (idx == 0) idx = 1;    // L'array dei cmdset  1-based
        Array.Resize(ref Dst.CmdLin, idx + CmdLin.Length - 1);
        for (int i = 1; i < CmdLin.Length; i++)
        {
          ACommand c = CmdLin[i];
          if (c == null)
            continue;
          //
          if (c.IdxForm > 0)
            c.IdxForm += MainFrm.CompOwner.iFrmOffs.intValue();   // Shifto le referenze alle form
          //
          c.Index = idx;
          Dst.CmdLin[idx++] = c;
        }
      }
      else  // Il componente  gi stato inizializzato in precedenza... infilo i comandi dove li ho messi l'ultima volta
      {
        // L'array di mio padre potrebbe essere pi grande del mio se i componenti sono "dinamici" e vengono caricati in un 
        // ordine diverso da quello precedente (es: app->cmp1->cmp2 e poi, in una sessione differente app->cmp2->cmp1)
        Array.Resize(ref Dst.CmdLin, Math.Max(CmdLin.Length, Dst.CmdLin.Length));
        Array.Copy(CmdLin, MyCmdHandler.CMD_OFFSET + 1, Dst.CmdLin, MyCmdHandler.CMD_OFFSET + 1, CmdLin.Length - (MyCmdHandler.CMD_OFFSET + 1));
      }
      //
      // Se non l'ho ancora fatto shifto tutte le costanti contenute in MyGlb
      if (MyCmdHandler.CMD_OFFSET == 0)
      {
        // Il mio offset  il numero di comandi di tutti i miei padri...
        // Per calcolarlo guardo a dove  finito il mio primo comando (l'elemento 0  NULL)
        // Quello  l'offset dei miei comandi rispetto all'array che contiene tutti i comandi di tutti i componenti
        MyCmdHandler.CMD_OFFSET = CmdLin[1].Index - 1;
        //
        String tn = (MainFrm.CompNameSpace != null ? MainFrm.CompNameSpace + "." : "");
        FieldInfo[] props = Assembly.GetExecutingAssembly().GetType(tn + "MyGlb").GetFields();
        for (int j = 0; j < props.Length; j++)
        {
          FieldInfo p = props[j];
          if (p.Name.StartsWith("CMD_") || p.Name.Equals("MAX_COMMANDS"))
            p.SetValue(null, ((int)p.GetValue(null)) + MyCmdHandler.CMD_OFFSET);
        }
      }
    }
  }
}
