// **********************************************
// Web Entry Point (session handler)
// Instant WEB Application: www.progamma.com
// Project : PRJ Handling Debug
// **********************************************
using System;
using System.Text;
using System.Collections;
using System.IO;
using System.Web;
using System.IO.Compression;
using Microsoft.Win32;
using com.progamma.ids;
using com.progamma;
using com.progamma.doc;
using CIDREObj = com.progamma.idre.CIDREObj;
using IDREGlb = com.progamma.idre.IDREGlb;

// **********************************************
// Classe base della servlet
// **********************************************
[Serializable]
public sealed class MyWebEntryPoint : WebEntryPoint
{
  public override WebEntryPoint CreateWEP() { return new MyWebEntryPoint(Parent()); }

  bool PreDesktopRendered = true;   // Pre-Inizializzazione: serve per bypassare il problema dell'F5 con videata di login (vedi NPQ 955)
  public MyWebEntryPoint MainFrm;

  //
  // Definition of Database Objects
  //
  // Definition of External Components
  //
  // Definition of Global Variables
  //  


  // **********************************************
  // Setting Global panel flags
  // **********************************************
  public void InitPanelGlbFlags()
  {
    GlbPanelFlags = MyGlb.PAN_USERETURN | MyGlb.PAN_CANSORT;
    if (MyParamsObj().PanelsUseTab.Equals("YES")) GlbPanelFlags = GlbPanelFlags | MyGlb.PAN_USETAB;
    if (MyParamsObj().PanelsSortAlways.Equals("YES")) GlbPanelFlags = GlbPanelFlags | MyGlb.PAN_SORTALWAYS;
    if (MyParamsObj().PanelNewInsertMode.Equals("YES")) GlbPanelFlags = GlbPanelFlags | MyGlb.PAN_NEWINSMODE;
    if (MyParamsObj().PanelFreezeWhenHidden.Equals("YES")) GlbPanelFlags = GlbPanelFlags | Glb.PAN_FREEWHENHID;
    //
    WelcomeURL = "qhelp_eng.htm";
    LogoffURL = "http://www.progamma.com";
    set_Image("");
    //
    try
    {
      iAccentColor = Convert.ToInt32("0081C2", 16);
      // Inverto la parte rossa e blu
      iAccentColor = ((iAccentColor & 0xFF0000) >> 16) + (iAccentColor & 0x00FF00) + ((iAccentColor & 0x0000FF) << 16);
    }
    catch (Exception)
    {
    }
  }


  // **********************************************
  // Logging a Message
  // **********************************************
  public override void log(String Message)
  {
    if (CompOwner == null)
    {
      HttpContext ctx = (HttpContext.Current != null ? HttpContext.Current : Parent());
      ((IDHttpHandler)ctx.CurrentHandler).log(Message);
      Console.Error.WriteLine(Message);
    }
    else
      CompOwner.log(Message);
  }


  // ************************************************
  // Determina il MimeType a partire dall'estensione
  // ************************************************
  public override String GetMimeType(String ext)
  {
    try
    {
      RegistryKey rk = Registry.ClassesRoot.OpenSubKey("." + ext.ToLower());
      Object mt = (rk == null ? null : rk.GetValue("Content Type"));
      if (mt != null)
        return (String)mt;
      //
      RegistryKey AllCntTypes = Registry.ClassesRoot.OpenSubKey("MIME\\Database\\Content Type");
      foreach (String cnttyp in AllCntTypes.GetSubKeyNames())
      {
        RegistryKey cnttypKey = Registry.ClassesRoot.OpenSubKey("MIME\\Database\\Content Type\\" + cnttyp);
        Object exttyp = cnttypKey.GetValue("Extension");
        if (exttyp != null && (String)exttyp == ext)
          return cnttyp;
      }
    }
    catch (Exception) {}
    //
    // Non ho trovato... provo con la mia mappa
    return GetMimeFromExtension(ext);
  }


  // **********************************************
  // La sessione sta per essere attivata dopo la deserializzazione!
  // **********************************************
  public void SetParent(HttpContext p)
  {
    set_Parent(p);
    IMDB.Reset();
  }


  // **********************************************
  // Una nuova sessione  iniziata!
  // **********************************************
  public MyWebEntryPoint(HttpContext p)
    : base()
  {
    MainFrm = this;
    //
    RevNum = 6;
    IDVer = "13.5.5800";
    iParamsObj = new MyParams();
    //
    // Se non sono un componente
    if (GetType().FullName.IndexOf('.') == -1)
    {
      // Se ci sono componenti in fase di inizializzazione non posso proseguire
      if (cmpInited == 1)
      {
        // I componenti stanno per essere inizializzati, aspetto che sia tutto finito
        DateTime dt = DateTime.Now;
        while (cmpInited != 2)
        {
          System.Threading.Thread.Sleep(50);
          //
          // Non aspetto all'infinito... se entro 5 secondi non finisce... forse  morto... ci provo io
          // e lascio entrare gli altri
          TimeSpan ts = (DateTime.Now - dt);
          if (ts.TotalSeconds > 5)
            cmpInited = 2;
        }
      }
      else if (cmpInited == 0)    // Se non sono ancora stati inizializzati i componenti, ci penso io
        cmpInited = 1;
    }
    //
    set_Parent(p);
    CmdObj = new MyCmdHandler(this);
    IndObj = new MyIndHandler(this);
    TimerObj = new MyTimerHandler(this);
    IwImdb = new MyImdbInit(this);
    RTCObj = new RTCEngine(this);
    RTCObj.AppGuid = "0DC03ED7-07F6-4CEA-9AAC-C69A27DE5CEB";
    VoiceObj = new VoiceHandler(this);
    IMDB = IwImdb.IMDB;
    //
    // Creo le ScreenZone
    if (UseZones())
    {
      ScreenZone sz = new ScreenZone(this, 0, Glb.FORMDOCK_LEFT, Glb.TABOR_LEFT);
      screenZones.Add(sz);
      sz = new ScreenZone(this, 1, Glb.FORMDOCK_RIGHT, Glb.TABOR_RIGHT);
      screenZones.Add(sz);
      sz = new ScreenZone(this, 2, Glb.FORMDOCK_TOP, Glb.TABOR_TOP);
      screenZones.Add(sz);
      sz = new ScreenZone(this, 3, Glb.FORMDOCK_BOTTOM, Glb.TABOR_BOTTOM);
      screenZones.Add(sz);
    }
    //
    // Creation of Database Objects
    //
    GZipEnabled = 1; // unknown
    ResBuf = new StringBuilder(20 * 1024);
    RDStatus = MyGlb.RD_DISABLED;
    //
    set_Caption(new IDVariant("Handling Debug"));
    InitPanelGlbFlags();
    IntSesNumber = (int)(Glb.Random.NextDouble() * 2147483647);
    UseRD = ParamsObj().UseRD;
    //
    // Initialize Visual Attributes
    //
    VisAttrObj v = null;
    //
    DTTObj.Init (5, 3, false, false, false, this);
    //
    VisualStyleList.set_Size(MyGlb.MAX_VISATTR);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_DEFAPANESTYL, v);
    v.iGuid = "2CBA05B6-623A-11D4-BDF0-301B4CC10101";
    v.set_Alignment(1, 1);
    v.set_Alignment(2, 1);
    v.set_Alignment(3, 1);
    v.set_ControlType(1);
    v.set_Mask("");
    v.set_Cursor("");
    v.set_RowOffset(0);
    v.set_HeaderOffset(4);
    v.set_Flags(786689);
    v.set_Color(1, 2105376);
    v.set_GradCol(1, -1);
    v.set_GradDir(1, 1);
    v.set_Opacity(1, 100);
    v.set_Color(2, 3952720);
    v.set_GradCol(2, -1);
    v.set_GradDir(2, 1);
    v.set_Opacity(2, 100);
    v.set_Color(3, 8998144);
    v.set_GradCol(3, -1);
    v.set_GradDir(3, 1);
    v.set_Opacity(3, 100);
    v.set_Color(4, 16777215);
    v.set_GradCol(4, -1);
    v.set_GradDir(4, 1);
    v.set_Opacity(4, 100);
    v.set_Color(5, 16777215);
    v.set_GradCol(5, -1);
    v.set_GradDir(5, 1);
    v.set_Opacity(5, 100);
    v.set_Color(6, 16054778);
    v.set_GradCol(6, -1);
    v.set_GradDir(6, 1);
    v.set_Opacity(6, 100);
    v.set_Color(7, 16382457);
    v.set_GradCol(7, 15658219);
    v.set_GradDir(7, 3);
    v.set_Opacity(7, 100);
    v.set_Color(8, 16777215);
    v.set_GradCol(8, 15658219);
    v.set_GradDir(8, 3);
    v.set_Opacity(8, 100);
    v.set_Color(9, 16446446);
    v.set_GradCol(9, -1);
    v.set_GradDir(9, 1);
    v.set_Opacity(9, 100);
    v.set_Color(10, -1);
    v.set_GradCol(10, -1);
    v.set_GradDir(10, 1);
    v.set_Opacity(10, 100);
    v.set_Color(11, 13547166);
    v.set_GradCol(11, -1);
    v.set_GradDir(11, 1);
    v.set_Opacity(11, 100);
    v.set_Color(12, -1);
    v.set_GradCol(12, -1);
    v.set_GradDir(12, 1);
    v.set_Opacity(12, 100);
    v.set_Color(13, 8998144);
    v.set_GradCol(13, -1);
    v.set_GradDir(13, 1);
    v.set_Opacity(13, 100);
    v.set_Color(14, 14811106);
    v.set_GradCol(14, -1);
    v.set_GradDir(14, 1);
    v.set_Opacity(14, 100);
    v.set_Color(15, 15856627);
    v.set_GradCol(15, -1);
    v.set_GradDir(15, 1);
    v.set_Opacity(15, 100);
    v.set_Color(16, 15658477);
    v.set_GradCol(16, -1);
    v.set_GradDir(16, 1);
    v.set_Opacity(16, 100);
    v.set_Color(17, -1);
    v.set_GradCol(17, -1);
    v.set_GradDir(17, 1);
    v.set_Opacity(17, 100);
    v.set_Color(18, -1);
    v.set_GradCol(18, -1);
    v.set_GradDir(18, 1);
    v.set_Opacity(18, 100);
    v.set_Color(19, -1);
    v.set_GradCol(19, -1);
    v.set_GradDir(19, 1);
    v.set_Opacity(19, 100);
    v.set_Color(20, -1);
    v.set_GradCol(20, -1);
    v.set_GradDir(20, 1);
    v.set_Opacity(20, 100);
    v.set_Color(21, -1);
    v.set_GradCol(21, -1);
    v.set_GradDir(21, 1);
    v.set_Opacity(21, 100);
    v.set_Color(22, 16777215);
    v.set_GradCol(22, -1);
    v.set_GradDir(22, 1);
    v.set_Opacity(22, 100);
    v.set_Color(23, 16777215);
    v.set_GradCol(23, -1);
    v.set_GradDir(23, 1);
    v.set_Opacity(23, 100);
    v.set_BorderType(1, 4);
    v.set_BorderType(2, 4);
    v.set_BorderType(3, 4);
    v.set_BorderType(4, 1);
    v.set_BorderType(5, 2);
    v.set_BorderType(6, 4);
    v.set_Font(1, "Tahoma,,8");
    v.set_Font(2, "Tahoma,,8");
    v.set_Font(3, "Tahoma,B,8");
    v.set_Font(4, "Tahoma,,8");
    v.set_Font(5, "Tahoma,,8");
    v.set_Font(6, "Tahoma,,8");
    v.set_HorizontalScale(0);
    v.set_LetterSpacing(0);
    v.set_WordSpacing(0);
    v.set_CBorColor(1, 13684944);
    v.set_CBorWidth(1, 1);
    v.set_CBorType(1, 1);
    v.set_CBorPadding(1, 1);
    v.set_CBorColor(2, 13684944);
    v.set_CBorWidth(2, 0);
    v.set_CBorType(2, 1);
    v.set_CBorPadding(2, 1);
    v.set_CBorColor(3, 13684944);
    v.set_CBorWidth(3, 1);
    v.set_CBorType(3, 1);
    v.set_CBorPadding(3, 1);
    v.set_CBorColor(4, 13684944);
    v.set_CBorWidth(4, 0);
    v.set_CBorType(4, 1);
    v.set_CBorPadding(4, 1);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_PRIMAKEYFIEL, v);
    v.iGuid = "2CBA05B7-623A-11D4-BDF0-301B4CC10101";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_Color(1, 139);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_FOREIKEYFIEL, v);
    v.iGuid = "2CBA05B8-623A-11D4-BDF0-301B4CC10101";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_BLOCKEFIELDS, v);
    v.iGuid = "2CBA05BA-623A-11D4-BDF0-301B4CC10101";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_Color(1, 8355711);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_LOOKUPFIELDS, v);
    v.iGuid = "2CBA05BB-623A-11D4-BDF0-301B4CC10101";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_Color(4, 16446446);
    v.set_Color(5, 16446446);
    v.set_Color(9, 16051431);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_LABELFIELD, v);
    v.iGuid = "0AA3D5EF-9931-11D4-8EBD-6C2303000000";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_Flags(917761);
    v.set_Color(4, -1);
    v.set_Color(5, -1);
    v.set_Color(9, -1);
    v.set_Color(15, -1);
    v.set_Color(16, -1);
    v.set_Color(22, -1);
    v.set_Color(23, -1);
    v.set_BorderType(6, 1);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_HYPERLIFIELD, v);
    v.iGuid = "D4423A16-E154-11D4-9021-88F920000000";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_LABELFIELD));
    v.set_Flags(4063489);
    v.set_Color(1, 16711680);
    v.set_Font(1, "Tahoma,U,8");
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_IMAGEFIELD, v);
    v.iGuid = "0C892579-65CA-494B-B7D0-B06FD2CA39FC";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_LABELFIELD));
    v.set_Flags(1966337);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_COMMANBUTTO1, v);
    v.iGuid = "D47A9B39-CB14-11D5-93C0-6A9CBE000000";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_Alignment(1, 3);
    v.set_ControlType(6);
    v.set_Flags(3539201);
    v.set_Color(4, -1);
    v.set_Color(5, -1);
    v.set_Color(9, -1);
    v.set_Color(14, -1);
    v.set_Color(15, -1);
    v.set_Color(16, -1);
    v.set_Color(22, -1);
    v.set_Color(23, -1);
    v.set_BorderType(1, 1);
    v.set_BorderType(6, 1);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_CHECKSTYLE, v);
    v.iGuid = "622C602E-9B5A-11D4-8ECE-323F96000000";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_ControlType(4);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_PASSWORSTYLE, v);
    v.iGuid = "D47A9B2C-CB14-11D5-93C0-6A9CBE000000";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_Flags(852225);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_RADIOSTYLE, v);
    v.iGuid = "DF846B65-D41A-444D-B05F-1378C91F7FC9";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_ControlType(5);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_NORMALFIELDS, v);
    v.iGuid = "2CBA05B9-623A-11D4-BDF0-301B4CC10101";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_HTMLEDITSTYL, v);
    v.iGuid = "8A9099EE-0441-46D1-959A-474AB1390B7F";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAPANESTYL));
    v.set_ControlType(7);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_DEFAREPOSTYL, v);
    v.iGuid = "EFA25462-6C01-4AB0-A6CA-0A083C3ACE46";
    v.set_Alignment(1, 1);
    v.set_Alignment(2, 1);
    v.set_Alignment(3, 1);
    v.set_ControlType(1);
    v.set_Mask("");
    v.set_Cursor("");
    v.set_RowOffset(3);
    v.set_HeaderOffset(4);
    v.set_Flags(1);
    v.set_Color(1, 0);
    v.set_GradCol(1, -1);
    v.set_GradDir(1, 1);
    v.set_Opacity(1, 100);
    v.set_Color(2, 0);
    v.set_GradCol(2, -1);
    v.set_GradDir(2, 1);
    v.set_Opacity(2, 100);
    v.set_Color(3, 0);
    v.set_GradCol(3, -1);
    v.set_GradDir(3, 1);
    v.set_Opacity(3, 100);
    v.set_Color(4, -1);
    v.set_GradCol(4, -1);
    v.set_GradDir(4, 1);
    v.set_Opacity(4, 100);
    v.set_Color(5, -1);
    v.set_GradCol(5, -1);
    v.set_GradDir(5, 1);
    v.set_Opacity(5, 100);
    v.set_Color(6, -1);
    v.set_GradCol(6, -1);
    v.set_GradDir(6, 1);
    v.set_Opacity(6, 100);
    v.set_Color(7, -1);
    v.set_GradCol(7, -1);
    v.set_GradDir(7, 1);
    v.set_Opacity(7, 100);
    v.set_Color(8, -1);
    v.set_GradCol(8, -1);
    v.set_GradDir(8, 1);
    v.set_Opacity(8, 100);
    v.set_Color(9, -1);
    v.set_GradCol(9, -1);
    v.set_GradDir(9, 1);
    v.set_Opacity(9, 100);
    v.set_Color(10, -1);
    v.set_GradCol(10, -1);
    v.set_GradDir(10, 1);
    v.set_Opacity(10, 100);
    v.set_Color(11, 8355711);
    v.set_GradCol(11, -1);
    v.set_GradDir(11, 1);
    v.set_Opacity(11, 100);
    v.set_Color(12, 12632256);
    v.set_GradCol(12, -1);
    v.set_GradDir(12, 1);
    v.set_Opacity(12, 100);
    v.set_Color(13, 0);
    v.set_GradCol(13, -1);
    v.set_GradDir(13, 1);
    v.set_Opacity(13, 100);
    v.set_Color(14, 12648384);
    v.set_GradCol(14, -1);
    v.set_GradDir(14, 1);
    v.set_Opacity(14, 100);
    v.set_Color(15, 12632256);
    v.set_GradCol(15, -1);
    v.set_GradDir(15, 1);
    v.set_Opacity(15, 100);
    v.set_Color(16, 14737632);
    v.set_GradCol(16, -1);
    v.set_GradDir(16, 1);
    v.set_Opacity(16, 100);
    v.set_Color(17, 14745599);
    v.set_GradCol(17, -1);
    v.set_GradDir(17, 1);
    v.set_Opacity(17, 100);
    v.set_Color(18, -1);
    v.set_GradCol(18, -1);
    v.set_GradDir(18, 1);
    v.set_Opacity(18, 100);
    v.set_Color(19, -1);
    v.set_GradCol(19, -1);
    v.set_GradDir(19, 1);
    v.set_Opacity(19, 100);
    v.set_Color(20, -1);
    v.set_GradCol(20, -1);
    v.set_GradDir(20, 1);
    v.set_Opacity(20, 100);
    v.set_Color(21, -1);
    v.set_GradCol(21, -1);
    v.set_GradDir(21, 1);
    v.set_Opacity(21, 100);
    v.set_Color(22, -1);
    v.set_GradCol(22, -1);
    v.set_GradDir(22, 1);
    v.set_Opacity(22, 100);
    v.set_Color(23, -1);
    v.set_GradCol(23, -1);
    v.set_GradDir(23, 1);
    v.set_Opacity(23, 100);
    v.set_BorderType(1, 1);
    v.set_BorderType(2, 4);
    v.set_BorderType(3, 4);
    v.set_BorderType(4, 4);
    v.set_BorderType(5, 7);
    v.set_BorderType(6, 4);
    v.set_Font(1, "Tahoma,,8");
    v.set_Font(2, "Tahoma,,8");
    v.set_Font(3, "Tahoma,,8");
    v.set_Font(4, "Tahoma,,8");
    v.set_Font(5, "Tahoma,,8");
    v.set_Font(6, "Tahoma,,8");
    v.set_HorizontalScale(0);
    v.set_LetterSpacing(0);
    v.set_WordSpacing(0);
    v.set_CBorColor(1, 13684944);
    v.set_CBorWidth(1, 1);
    v.set_CBorType(1, 1);
    v.set_CBorPadding(1, 1);
    v.set_CBorColor(2, 13684944);
    v.set_CBorWidth(2, 1);
    v.set_CBorType(2, 1);
    v.set_CBorPadding(2, 1);
    v.set_CBorColor(3, 13684944);
    v.set_CBorWidth(3, 1);
    v.set_CBorType(3, 1);
    v.set_CBorPadding(3, 1);
    v.set_CBorColor(4, 13684944);
    v.set_CBorWidth(4, 1);
    v.set_CBorType(4, 1);
    v.set_CBorPadding(4, 1);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_COMMANBUTTON, v);
    v.iGuid = "35A1432C-83AD-4FC0-AC73-F1975EC5B17D";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAREPOSTYL));
    v.set_Alignment(1, 3);
    v.set_ControlType(6);
    v.set_Flags(2490625);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_HYPERLINK, v);
    v.iGuid = "79F9869B-2BD8-4446-9CFF-498C56A82BD3";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAREPOSTYL));
    v.set_Flags(2490625);
    v.set_Color(1, 16711680);
    v.set_Font(1, "Tahoma,U,8");
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_DEFAMOBISTYL, v);
    v.iGuid = "43A16029-32EA-4042-B82D-E48398C51D36";
    v.set_Alignment(1, 1);
    v.set_Alignment(2, 1);
    v.set_Alignment(3, 1);
    v.set_ControlType(1);
    v.set_Mask("");
    v.set_Cursor("");
    v.set_RowOffset(0);
    v.set_HeaderOffset(0);
    v.set_Flags(786689);
    v.set_Color(1, 0);
    v.set_GradCol(1, -1);
    v.set_GradDir(1, 1);
    v.set_Opacity(1, 100);
    v.set_Color(2, 8419441);
    v.set_GradCol(2, -1);
    v.set_GradDir(2, 1);
    v.set_Opacity(2, 100);
    v.set_Color(3, 7099980);
    v.set_GradCol(3, -1);
    v.set_GradDir(3, 1);
    v.set_Opacity(3, 100);
    v.set_Color(4, 16777215);
    v.set_GradCol(4, -1);
    v.set_GradDir(4, 1);
    v.set_Opacity(4, 100);
    v.set_Color(5, 16777215);
    v.set_GradCol(5, -1);
    v.set_GradDir(5, 1);
    v.set_Opacity(5, 100);
    v.set_Color(6, -1);
    v.set_GradCol(6, -1);
    v.set_GradDir(6, 1);
    v.set_Opacity(6, 100);
    v.set_Color(7, 16382457);
    v.set_GradCol(7, 12102567);
    v.set_GradDir(7, 3);
    v.set_Opacity(7, 100);
    v.set_Color(8, 16777215);
    v.set_GradCol(8, 15658219);
    v.set_GradDir(8, 3);
    v.set_Opacity(8, 100);
    v.set_Color(9, 15790320);
    v.set_GradCol(9, -1);
    v.set_GradDir(9, 1);
    v.set_Opacity(9, 100);
    v.set_Color(10, -1);
    v.set_GradCol(10, -1);
    v.set_GradDir(10, 1);
    v.set_Opacity(10, 100);
    v.set_Color(11, 13421772);
    v.set_GradCol(11, -1);
    v.set_GradDir(11, 1);
    v.set_Opacity(11, 100);
    v.set_Color(12, 16777215);
    v.set_GradCol(12, -1);
    v.set_GradDir(12, 1);
    v.set_Opacity(12, 100);
    v.set_Color(13, 7099980);
    v.set_GradCol(13, -1);
    v.set_GradDir(13, 1);
    v.set_Opacity(13, 100);
    v.set_Color(14, 14811106);
    v.set_GradCol(14, -1);
    v.set_GradDir(14, 1);
    v.set_Opacity(14, 100);
    v.set_Color(15, 16777215);
    v.set_GradCol(15, -1);
    v.set_GradDir(15, 1);
    v.set_Opacity(15, 100);
    v.set_Color(16, 15790320);
    v.set_GradCol(16, -1);
    v.set_GradDir(16, 1);
    v.set_Opacity(16, 100);
    v.set_Color(17, -1);
    v.set_GradCol(17, -1);
    v.set_GradDir(17, 1);
    v.set_Opacity(17, 100);
    v.set_Color(18, -1);
    v.set_GradCol(18, -1);
    v.set_GradDir(18, 1);
    v.set_Opacity(18, 100);
    v.set_Color(19, -1);
    v.set_GradCol(19, -1);
    v.set_GradDir(19, 1);
    v.set_Opacity(19, 100);
    v.set_Color(20, -1);
    v.set_GradCol(20, -1);
    v.set_GradDir(20, 1);
    v.set_Opacity(20, 100);
    v.set_Color(21, -1);
    v.set_GradCol(21, -1);
    v.set_GradDir(21, 1);
    v.set_Opacity(21, 100);
    v.set_Color(22, 16777215);
    v.set_GradCol(22, -1);
    v.set_GradDir(22, 1);
    v.set_Opacity(22, 100);
    v.set_Color(23, 16777215);
    v.set_GradCol(23, -1);
    v.set_GradDir(23, 1);
    v.set_Opacity(23, 100);
    v.set_BorderType(1, 9);
    v.set_BorderType(2, 9);
    v.set_BorderType(3, 4);
    v.set_BorderType(4, 9);
    v.set_BorderType(5, 4);
    v.set_BorderType(6, 9);
    v.set_Font(1, "Arial,,13");
    v.set_Font(2, "Arial,B,13");
    v.set_Font(3, "Arial,B,13");
    v.set_Font(4, "Arial,,13");
    v.set_Font(5, "Arial,,13");
    v.set_Font(6, "Tahoma,,8");
    v.set_HorizontalScale(0);
    v.set_LetterSpacing(0);
    v.set_WordSpacing(0);
    v.set_CBorColor(1, 13684944);
    v.set_CBorWidth(1, 1);
    v.set_CBorType(1, 1);
    v.set_CBorPadding(1, 32);
    v.set_CBorColor(2, 13684944);
    v.set_CBorWidth(2, 0);
    v.set_CBorType(2, 1);
    v.set_CBorPadding(2, 40);
    v.set_CBorColor(3, 13684944);
    v.set_CBorWidth(3, 1);
    v.set_CBorType(3, 1);
    v.set_CBorPadding(3, 32);
    v.set_CBorColor(4, 13684944);
    v.set_CBorWidth(4, 0);
    v.set_CBorType(4, 1);
    v.set_CBorPadding(4, 40);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_PRIKEYFIEMOB, v);
    v.iGuid = "BAE49E85-8C0D-4222-A404-83C5DB962F89";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_Flags(1835265);
    v.set_Color(1, 139);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_FORKEYFIEMOB, v);
    v.iGuid = "81F8B2FB-DA5E-4C79-95D1-D741C8BFD85B";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_BLOCKFIELMOB, v);
    v.iGuid = "99ED52E9-6C42-4BF2-851D-7D2774698FF8";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_Color(1, 8355711);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_LOOKUFIELMOB, v);
    v.iGuid = "F15E3F4D-1C3F-44B5-AB5C-6F28B23F2D90";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_LABELFIELMOB, v);
    v.iGuid = "A8B47B02-5E37-4978-835F-4275B85F4327";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_Alignment(1, 3);
    v.set_Flags(1966337);
    v.set_Color(1, 7099980);
    v.set_Color(4, -1);
    v.set_Color(15, -1);
    v.set_Color(16, -1);
    v.set_Color(22, -1);
    v.set_Color(23, -1);
    v.set_BorderType(6, 1);
    v.set_Font(1, "Arial,,11");
    v.set_CBorPadding(2, 0);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_HYPERFIELMOB, v);
    v.iGuid = "B54E087A-BCEC-41AE-8ED5-91E87FD0990C";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_LABELFIELMOB));
    v.set_Flags(4063489);
    v.set_Color(1, 16711680);
    v.set_Font(1, "Arial,U,11");
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_IMAGEFIELMOB, v);
    v.iGuid = "126D21F9-8F31-438E-A18E-C39BBC3AD0DF";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_LABELFIELMOB));
    v.set_Alignment(1, 1);
    v.set_Flags(2883841);
    v.set_Color(1, 0);
    v.set_Color(4, 16777215);
    v.set_Color(15, 16777215);
    v.set_Color(16, 15790320);
    v.set_Color(22, 16777215);
    v.set_Color(23, 16777215);
    v.set_BorderType(6, 9);
    v.set_Font(1, "Arial,,13");
    v.set_CBorPadding(2, 40);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_COMMABUTTMOB, v);
    v.iGuid = "959C76FD-D3BB-467D-BFD0-54045A9EAB32";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_Alignment(1, 3);
    v.set_ControlType(6);
    v.set_Flags(3539201);
    v.set_Color(1, 8869177);
    v.set_Color(14, -1);
    v.set_Color(15, -1);
    v.set_Color(16, -1);
    v.set_Color(22, -1);
    v.set_Color(23, -1);
    v.set_Font(1, "Arial,B,11");
    v.set_CBorWidth(1, 0);
    v.set_CBorWidth(3, 0);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_CHECKSTYLMOB, v);
    v.iGuid = "F3702BE0-4837-49AC-98C9-B3461F813EFE";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_ControlType(4);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_PASSWSTYLMOB, v);
    v.iGuid = "DDFFB40A-6D1D-457F-8098-2ACBBD2A7937";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_Flags(852225);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_RADIOSTYLMOB, v);
    v.iGuid = "01F3CF34-A192-4B9E-AEBA-B66EA69D5319";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_ControlType(5);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_NORMAFIELMOB, v);
    v.iGuid = "F5ED2E22-0A7B-49B1-821B-765AFEE9F2B6";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_HTMEDISTYMOB, v);
    v.iGuid = "1C77869E-E35A-48C8-AAC7-DF0A1B06DB87";
    v.set_Derived(VisualStyleList.VisualAttribute(MyGlb.VIS_DEFAMOBISTYL));
    v.set_ControlType(7);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_SIMPLICITY, v);
    v.iGuid = "08FD91B7-5C04-4582-B0D5-2EB995A3A0B0";
    v.set_Alignment(1, 1);
    v.set_Alignment(2, 1);
    v.set_Alignment(3, 1);
    v.set_ControlType(1);
    v.set_Mask("");
    v.set_Cursor("");
    v.set_RowOffset(0);
    v.set_HeaderOffset(4);
    v.set_Flags(1835265);
    v.set_Color(1, 3368448);
    v.set_GradCol(1, -1);
    v.set_GradDir(1, 1);
    v.set_Opacity(1, 100);
    v.set_Color(2, 3368448);
    v.set_GradCol(2, -1);
    v.set_GradDir(2, 1);
    v.set_Opacity(2, 100);
    v.set_Color(3, 3368448);
    v.set_GradCol(3, -1);
    v.set_GradDir(3, 1);
    v.set_Opacity(3, 100);
    v.set_Color(4, 16777215);
    v.set_GradCol(4, -1);
    v.set_GradDir(4, 1);
    v.set_Opacity(4, 100);
    v.set_Color(5, 16777215);
    v.set_GradCol(5, -1);
    v.set_GradDir(5, 1);
    v.set_Opacity(5, 100);
    v.set_Color(6, 16777215);
    v.set_GradCol(6, -1);
    v.set_GradDir(6, 1);
    v.set_Opacity(6, 100);
    v.set_Color(7, 14413275);
    v.set_GradCol(7, -1);
    v.set_GradDir(7, 1);
    v.set_Opacity(7, 100);
    v.set_Color(8, 14413275);
    v.set_GradCol(8, -1);
    v.set_GradDir(8, 1);
    v.set_Opacity(8, 100);
    v.set_Color(9, 16577775);
    v.set_GradCol(9, -1);
    v.set_GradDir(9, 1);
    v.set_Opacity(9, 100);
    v.set_Color(10, 16706784);
    v.set_GradCol(10, -1);
    v.set_GradDir(10, 1);
    v.set_Opacity(10, 100);
    v.set_Color(11, 6736896);
    v.set_GradCol(11, -1);
    v.set_GradDir(11, 1);
    v.set_Opacity(11, 100);
    v.set_Color(12, -1);
    v.set_GradCol(12, -1);
    v.set_GradDir(12, 1);
    v.set_Opacity(12, 100);
    v.set_Color(13, 3368448);
    v.set_GradCol(13, -1);
    v.set_GradDir(13, 1);
    v.set_Opacity(13, 100);
    v.set_Color(14, 14220504);
    v.set_GradCol(14, -1);
    v.set_GradDir(14, 1);
    v.set_Opacity(14, 100);
    v.set_Color(15, 15660535);
    v.set_GradCol(15, -1);
    v.set_GradDir(15, 1);
    v.set_Opacity(15, 100);
    v.set_Color(16, 15265778);
    v.set_GradCol(16, -1);
    v.set_GradDir(16, 1);
    v.set_Opacity(16, 100);
    v.set_Color(17, -1);
    v.set_GradCol(17, -1);
    v.set_GradDir(17, 1);
    v.set_Opacity(17, 100);
    v.set_Color(18, -1);
    v.set_GradCol(18, -1);
    v.set_GradDir(18, 1);
    v.set_Opacity(18, 100);
    v.set_Color(19, -1);
    v.set_GradCol(19, -1);
    v.set_GradDir(19, 1);
    v.set_Opacity(19, 100);
    v.set_Color(20, -1);
    v.set_GradCol(20, -1);
    v.set_GradDir(20, 1);
    v.set_Opacity(20, 100);
    v.set_Color(21, -1);
    v.set_GradCol(21, -1);
    v.set_GradDir(21, 1);
    v.set_Opacity(21, 100);
    v.set_Color(22, -1);
    v.set_GradCol(22, -1);
    v.set_GradDir(22, 1);
    v.set_Opacity(22, 100);
    v.set_Color(23, -1);
    v.set_GradCol(23, -1);
    v.set_GradDir(23, 1);
    v.set_Opacity(23, 100);
    v.set_BorderType(1, 4);
    v.set_BorderType(2, 4);
    v.set_BorderType(3, 4);
    v.set_BorderType(4, 1);
    v.set_BorderType(5, 2);
    v.set_BorderType(6, 4);
    v.set_Font(1, "Tahoma,,8");
    v.set_Font(2, "Tahoma,,8");
    v.set_Font(3, "Tahoma,B,8");
    v.set_Font(4, "Tahoma,,8");
    v.set_Font(5, "Tahoma,,8");
    v.set_Font(6, "Tahoma,,8");
    v.set_HorizontalScale(0);
    v.set_LetterSpacing(0);
    v.set_WordSpacing(0);
    v.set_CBorColor(1, 13684944);
    v.set_CBorWidth(1, 1);
    v.set_CBorType(1, 1);
    v.set_CBorPadding(1, 1);
    v.set_CBorColor(2, 13684944);
    v.set_CBorWidth(2, 1);
    v.set_CBorType(2, 1);
    v.set_CBorPadding(2, 1);
    v.set_CBorColor(3, 13684944);
    v.set_CBorWidth(3, 1);
    v.set_CBorType(3, 1);
    v.set_CBorPadding(3, 1);
    v.set_CBorColor(4, 13684944);
    v.set_CBorWidth(4, 1);
    v.set_CBorType(4, 1);
    v.set_CBorPadding(4, 1);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_CASUAL, v);
    v.iGuid = "4EF4E49D-B51C-4314-ABE0-10647F6866C4";
    v.set_Alignment(1, 1);
    v.set_Alignment(2, 1);
    v.set_Alignment(3, 1);
    v.set_ControlType(1);
    v.set_Mask("");
    v.set_Cursor("");
    v.set_RowOffset(0);
    v.set_HeaderOffset(4);
    v.set_Flags(786689);
    v.set_Color(1, 2105376);
    v.set_GradCol(1, -1);
    v.set_GradDir(1, 1);
    v.set_Opacity(1, 100);
    v.set_Color(2, 3952720);
    v.set_GradCol(2, -1);
    v.set_GradDir(2, 1);
    v.set_Opacity(2, 100);
    v.set_Color(3, 8998144);
    v.set_GradCol(3, -1);
    v.set_GradDir(3, 1);
    v.set_Opacity(3, 100);
    v.set_Color(4, 16777215);
    v.set_GradCol(4, -1);
    v.set_GradDir(4, 1);
    v.set_Opacity(4, 100);
    v.set_Color(5, 16777215);
    v.set_GradCol(5, -1);
    v.set_GradDir(5, 1);
    v.set_Opacity(5, 100);
    v.set_Color(6, 16777215);
    v.set_GradCol(6, -1);
    v.set_GradDir(6, 1);
    v.set_Opacity(6, 100);
    v.set_Color(7, 14085101);
    v.set_GradCol(7, -1);
    v.set_GradDir(7, 1);
    v.set_Opacity(7, 100);
    v.set_Color(8, 13166312);
    v.set_GradCol(8, -1);
    v.set_GradDir(8, 1);
    v.set_Opacity(8, 100);
    v.set_Color(9, 16511466);
    v.set_GradCol(9, -1);
    v.set_GradDir(9, 1);
    v.set_Opacity(9, 100);
    v.set_Color(10, -1);
    v.set_GradCol(10, -1);
    v.set_GradDir(10, 1);
    v.set_Opacity(10, 100);
    v.set_Color(11, 12164479);
    v.set_GradCol(11, -1);
    v.set_GradDir(11, 1);
    v.set_Opacity(11, 100);
    v.set_Color(12, 16777215);
    v.set_GradCol(12, -1);
    v.set_GradDir(12, 1);
    v.set_Opacity(12, 100);
    v.set_Color(13, 8998144);
    v.set_GradCol(13, -1);
    v.set_GradDir(13, 1);
    v.set_Opacity(13, 100);
    v.set_Color(14, 13695974);
    v.set_GradCol(14, -1);
    v.set_GradDir(14, 1);
    v.set_Opacity(14, 100);
    v.set_Color(15, 15660535);
    v.set_GradCol(15, -1);
    v.set_GradDir(15, 1);
    v.set_Opacity(15, 100);
    v.set_Color(16, 14871535);
    v.set_GradCol(16, -1);
    v.set_GradDir(16, 1);
    v.set_Opacity(16, 100);
    v.set_Color(17, -1);
    v.set_GradCol(17, -1);
    v.set_GradDir(17, 1);
    v.set_Opacity(17, 100);
    v.set_Color(18, -1);
    v.set_GradCol(18, -1);
    v.set_GradDir(18, 1);
    v.set_Opacity(18, 100);
    v.set_Color(19, -1);
    v.set_GradCol(19, -1);
    v.set_GradDir(19, 1);
    v.set_Opacity(19, 100);
    v.set_Color(20, -1);
    v.set_GradCol(20, -1);
    v.set_GradDir(20, 1);
    v.set_Opacity(20, 100);
    v.set_Color(21, -1);
    v.set_GradCol(21, -1);
    v.set_GradDir(21, 1);
    v.set_Opacity(21, 100);
    v.set_Color(22, -1);
    v.set_GradCol(22, -1);
    v.set_GradDir(22, 1);
    v.set_Opacity(22, 100);
    v.set_Color(23, -1);
    v.set_GradCol(23, -1);
    v.set_GradDir(23, 1);
    v.set_Opacity(23, 100);
    v.set_BorderType(1, 4);
    v.set_BorderType(2, 4);
    v.set_BorderType(3, 4);
    v.set_BorderType(4, 1);
    v.set_BorderType(5, 2);
    v.set_BorderType(6, 4);
    v.set_Font(1, "Tahoma,,8");
    v.set_Font(2, "Tahoma,,8");
    v.set_Font(3, "Tahoma,B,8");
    v.set_Font(4, "Tahoma,,8");
    v.set_Font(5, "Tahoma,,8");
    v.set_Font(6, "Tahoma,,8");
    v.set_HorizontalScale(0);
    v.set_LetterSpacing(0);
    v.set_WordSpacing(0);
    v.set_CBorColor(1, 13684944);
    v.set_CBorWidth(1, 1);
    v.set_CBorType(1, 1);
    v.set_CBorPadding(1, 1);
    v.set_CBorColor(2, 13684944);
    v.set_CBorWidth(2, 1);
    v.set_CBorType(2, 1);
    v.set_CBorPadding(2, 1);
    v.set_CBorColor(3, 13684944);
    v.set_CBorWidth(3, 1);
    v.set_CBorType(3, 1);
    v.set_CBorPadding(3, 1);
    v.set_CBorColor(4, 13684944);
    v.set_CBorWidth(4, 1);
    v.set_CBorType(4, 1);
    v.set_CBorPadding(4, 1);
    //
    v = new VisAttrObj();
    VisualStyleList.set_VisualAttribute(MyGlb.VIS_SEATTLE, v);
    v.iGuid = "DD74AA17-2B85-43FE-A1DC-6A7B7BF72AEE";
    v.set_Alignment(1, 1);
    v.set_Alignment(2, 1);
    v.set_Alignment(3, 1);
    v.set_ControlType(1);
    v.set_Mask("");
    v.set_Cursor("");
    v.set_RowOffset(0);
    v.set_HeaderOffset(4);
    v.set_Flags(786689);
    v.set_Color(1, 2105376);
    v.set_GradCol(1, -1);
    v.set_GradDir(1, 1);
    v.set_Opacity(1, 100);
    v.set_Color(2, 3952720);
    v.set_GradCol(2, -1);
    v.set_GradDir(2, 1);
    v.set_Opacity(2, 100);
    v.set_Color(3, 8998144);
    v.set_GradCol(3, -1);
    v.set_GradDir(3, 1);
    v.set_Opacity(3, 100);
    v.set_Color(4, 16777215);
    v.set_GradCol(4, -1);
    v.set_GradDir(4, 1);
    v.set_Opacity(4, 100);
    v.set_Color(5, 16777215);
    v.set_GradCol(5, -1);
    v.set_GradDir(5, 1);
    v.set_Opacity(5, 100);
    v.set_Color(6, 16054778);
    v.set_GradCol(6, -1);
    v.set_GradDir(6, 1);
    v.set_Opacity(6, 100);
    v.set_Color(7, 16382457);
    v.set_GradCol(7, 15658219);
    v.set_GradDir(7, 3);
    v.set_Opacity(7, 100);
    v.set_Color(8, 16777215);
    v.set_GradCol(8, 15658219);
    v.set_GradDir(8, 3);
    v.set_Opacity(8, 100);
    v.set_Color(9, 16446446);
    v.set_GradCol(9, -1);
    v.set_GradDir(9, 1);
    v.set_Opacity(9, 100);
    v.set_Color(10, -1);
    v.set_GradCol(10, -1);
    v.set_GradDir(10, 1);
    v.set_Opacity(10, 100);
    v.set_Color(11, 13547166);
    v.set_GradCol(11, -1);
    v.set_GradDir(11, 1);
    v.set_Opacity(11, 100);
    v.set_Color(12, -1);
    v.set_GradCol(12, -1);
    v.set_GradDir(12, 1);
    v.set_Opacity(12, 100);
    v.set_Color(13, 8998144);
    v.set_GradCol(13, -1);
    v.set_GradDir(13, 1);
    v.set_Opacity(13, 100);
    v.set_Color(14, 14811106);
    v.set_GradCol(14, -1);
    v.set_GradDir(14, 1);
    v.set_Opacity(14, 100);
    v.set_Color(15, 15856627);
    v.set_GradCol(15, -1);
    v.set_GradDir(15, 1);
    v.set_Opacity(15, 100);
    v.set_Color(16, 15658477);
    v.set_GradCol(16, -1);
    v.set_GradDir(16, 1);
    v.set_Opacity(16, 100);
    v.set_Color(17, -1);
    v.set_GradCol(17, -1);
    v.set_GradDir(17, 1);
    v.set_Opacity(17, 100);
    v.set_Color(18, -1);
    v.set_GradCol(18, -1);
    v.set_GradDir(18, 1);
    v.set_Opacity(18, 100);
    v.set_Color(19, -1);
    v.set_GradCol(19, -1);
    v.set_GradDir(19, 1);
    v.set_Opacity(19, 100);
    v.set_Color(20, -1);
    v.set_GradCol(20, -1);
    v.set_GradDir(20, 1);
    v.set_Opacity(20, 100);
    v.set_Color(21, -1);
    v.set_GradCol(21, -1);
    v.set_GradDir(21, 1);
    v.set_Opacity(21, 100);
    v.set_Color(22, 16777215);
    v.set_GradCol(22, -1);
    v.set_GradDir(22, 1);
    v.set_Opacity(22, 100);
    v.set_Color(23, 16777215);
    v.set_GradCol(23, -1);
    v.set_GradDir(23, 1);
    v.set_Opacity(23, 100);
    v.set_BorderType(1, 4);
    v.set_BorderType(2, 4);
    v.set_BorderType(3, 4);
    v.set_BorderType(4, 1);
    v.set_BorderType(5, 2);
    v.set_BorderType(6, 4);
    v.set_Font(1, "Tahoma,,8");
    v.set_Font(2, "Tahoma,,8");
    v.set_Font(3, "Tahoma,B,8");
    v.set_Font(4, "Tahoma,,8");
    v.set_Font(5, "Tahoma,,8");
    v.set_Font(6, "Tahoma,,8");
    v.set_HorizontalScale(0);
    v.set_LetterSpacing(0);
    v.set_WordSpacing(0);
    v.set_CBorColor(1, 13684944);
    v.set_CBorWidth(1, 1);
    v.set_CBorType(1, 1);
    v.set_CBorPadding(1, 1);
    v.set_CBorColor(2, 13684944);
    v.set_CBorWidth(2, 0);
    v.set_CBorType(2, 1);
    v.set_CBorPadding(2, 1);
    v.set_CBorColor(3, 13684944);
    v.set_CBorWidth(3, 1);
    v.set_CBorType(3, 1);
    v.set_CBorPadding(3, 1);
    v.set_CBorColor(4, 13684944);
    v.set_CBorWidth(4, 0);
    v.set_CBorType(4, 1);
    v.set_CBorPadding(4, 1);
    HelpFile = "";
    SetVSInited(1);
    //
    // Listing cached images
    //
    // Initializing WebServices URLs
    //
    SetHelpFile(HelpFile);
    ErrObj = new ErrorHandler(this);
    //
    // RD3
    RD3EntryPoint = "HandlingDebug.aspx";
    //
    if (!ParamsObj().Theme.Equals("seattle"))
      set_SideMenuWidth(163);
  }


  // **********************************************
  // La sessione  stata distrutta
  // **********************************************
  ~MyWebEntryPoint()
  {
    if (!AlreadyDestroyed)
      _WebEntryPoint();
  }

  public override void _WebEntryPoint()
  {
    try
    {
      // Comunico che sono gi stato distrutto
      AlreadyDestroyed = true;
      //
      TerminateApp();
      //
      // Chiamo il distruttore della mia classe base
      base._WebEntryPoint();
    }
    catch (Exception) { }
    finally
    {
      try
      {
        EndRequest();
      }
      catch (Exception) { }
    }
    //
    // Deleting Temp Files & PDF Files
    for (int i=0; i<TempFiles.Count; i++)
    {
      try
      {
        String s = (String)TempFiles[i];
        if (File.Exists(s))
          File.Delete(s);
      }
      catch (Exception)
      {
      }
    }
    for (int i = 0; i < PDFPrints.Count; i++)
    {
      try
      {
        PDFPrint p =(PDFPrint)PDFPrints[i]; 
        if (p.AutoDelete)
        {
          if (File.Exists(RealPath + "/" + p.PDFFile))
            File.Delete(RealPath + "/" + p.PDFFile);
        }
      }
      catch (Exception)
      {
      }
    }
  }


  // **********************************************
  // Componenti
  // **********************************************
  public override void ShiftFrmConst(int frmOfs) { MyGlb.ShiftFrmConst(frmOfs, this); }
  public override int IMDBOffset() { return MyImdbInit.IMDB_OFFSET; }
  public override void CreateComponents() 
  {
    // Creating components
    //
    // Creation of Global Variables
  }

  // **********************************************
  // Crea i ruoli del componente
  // **********************************************
  public override void CreateCompRoles()
  {
    // Se non l'ho ancora fatto, creo il ruolo
    if (RolObj == null)
    {
      IDConnection iDB = null;
      //
      RolObj = new MyRoles(this);
      //
      RTCObj.AppGuid = "0DC03ED7-07F6-4CEA-9AAC-C69A27DE5CEB";
      RTCObj.PrjGuid = "29291891-7C5E-448E-B89C-D81BC0722BA9";
      //
      // Chiamo il metodo base
      base.CreateCompRoles();
      //
      // Ho allineato tutti i componenti... ri-sincronizzo tutte le IMDB delle
      // variabili globali (ora tutte le strutture IMDB sono state mergiate tra loro)
      //
      // Passo ai miei figli
      foreach (WebEntryPoint cmp in CompList)
        cmp.CreateCompRoles();
    }
  }

  // **********************************************
  // Crea un'istanza di una classe DO a partire dal nome
  // e lo inserisce nella collection
  // **********************************************
  public override IDDocument CreateDocument(String ClassName)
  {
    IDDocument d = MyMDOInit.CreateDocument(ClassName);
    if (d == null)
    {
      // Io non ci sono riuscito... se ho dei componenti provo con loro
      foreach (WebEntryPoint cmp in CompList)
      {
        // Se questo componente ci riesce, ho finito
        d = cmp.CreateDocument(ClassName);
        if (d != null)
          break;
      }
    }
    if (d!=null)
      d.SetMainFrm(this, IMDB);
    return d;
  }

  public override bool CreateDocument(IDCollection IDColl, String ClassName)
  {
    try
    {
      IDDocument d = MyMDOInit.CreateDocument(ClassName);
      if (d == null)
      {
        // Io non ci sono riuscito... se ho dei componenti provo con loro
        foreach (WebEntryPoint cmp in CompList)
        {
          // Se questo componente ci riesce, ho finito
          if (cmp.CreateDocument(IDColl, ClassName))
            return true;
        }
        //
        // Nessun componente ne sa nulla... esco
        return false;
      }
      //
      d.SetMainFrm(MainFrm, IMDB);
      d.Init();
      d.set_Inserted(true);
      IDColl.Add(d);
      IDColl.MoveLast();
      d.OnInserting();
      if (d.Parent() != null)
        d.Parent().ReconnectChildren(9999);
      //
      // OK
      return true;
    }
    catch (Exception)
    {
      return false;
    }
  }

  // **********************************************
  // Crea un documento dall'assembly giusto
  // **********************************************
  public override IDDocument GetFromDNA(IDVariant DNA, bool FromCache, int NumLevel, bool SkipLoad) 
  {
    return IDDocument.GetFromDNA(DNA, FromCache, NumLevel, this, IMDB, SkipLoad);
  }
  

  // **********************************************
  // Gestisce una nuova richiesta
  // **********************************************
  public override void HandleRequest(HttpRequest req, HttpResponse resp)
  {
    lock (this)
    {
      TextWriter OldOutStream = null;
      TextWriter OldErrStream = null;
      //
      try
      {
        base.HandleRequest(req, resp);
        //
        if (req != null)
        {
          // Se la richiesta arriva dal frame RD, forzo l'ASCII encoding
          if (req["WCI"] == MyGlb.ITEM_RD)
            req.ContentEncoding = ASCIIEncoding.Default;
        }
        //
        String WebItem = null, WebEvent = null, UrlData = null;
        //
        // Prelevo il path base
        if (RealPath.Length == 0)
        {
          AppPath = System.Web.HttpRuntime.AppDomainAppVirtualPath;
          if (AppPath.EndsWith("/"))
            AppPath = AppPath.Substring(0, AppPath.Length - 1);
          //
          AppName = AppPath + "/HandlingDebug.aspx";
          //
          RealPath = System.Web.HttpRuntime.AppDomainAppPath;
          if (RealPath.EndsWith(Path.DirectorySeparatorChar.ToString()))
            RealPath = RealPath.Substring(0, RealPath.Length - 1);
          //
          try
          {
            // Se non ci sono le cartelle "temp" e "logs" le creo
            String dirTemp = RealPath + Path.DirectorySeparatorChar.ToString() + "temp";
            if (!Directory.Exists(dirTemp))
              Directory.CreateDirectory(dirTemp);
            //
            String dirLogs = RealPath + Path.DirectorySeparatorChar.ToString() + "logs";
            if (DTTObj.Level > 0 && !Directory.Exists(dirLogs))
              Directory.CreateDirectory(dirLogs);
          }
          catch (Exception) { }
          //
          RTCObj.LoadRTCConst();
          //
          // Ora che RTC  inizializzato, inizializzo anche l'error handler
          ErrObj.Init();
          //
          // Ora che RTC  inizializzato e conosco il path, inizializzo anche il voice handler
          VoiceObj.Init();
        }
        //
        // Se il debug  attivo (e non c' il trace attivo), ridirigo gli Stream di Out ed Error su files
        if (DTTObj.Level > 0 && !DTTObj.TraceEnabled)
        {
          try
          {
            TextWriter tw = CreateLogFile("Output");
            if (tw != null)
            {
              OldOutStream = Console.Out;
              Console.SetOut(tw);
            }
          }
          catch (Exception) {}
          //
          try
          {
            TextWriter tw = CreateLogFile("Error");
            if (tw != null)
            {
              OldErrStream = Console.Error;
              Console.SetError(tw);
            }
          }
          catch (Exception) {}
        }
        //
        Request = req;
        Response = resp;
        //
        DTTObj.AddRequest(this);
        //
        if (Request != null)
        {
          // Estraggo i parametri dall'URL
          ExtractUrlParams(Request);
          bool flCmd = true;
          if (ActiveForm() != null && ActiveForm().UploadingPanel != null)
            flCmd = false;
          if (UrlParams.Count == 0 && flCmd)
            ExtractUrlParams2(Request.Form["cmd"]);  // Provo con CMD (form-rd)
          if (UrlParams.Count == 0 && flCmd)
            ExtractUrlParams2(Request["ss"]);   // Provo con CMD proveniente da RD
        }
        else if (IsServerSession())
          ExtractUrlParams2(SessionQueryString);
        //
        if (RolObj == null)
        {
          RolObj = new MyRoles(this);
          //
          RTCObj.AppGuid = "0DC03ED7-07F6-4CEA-9AAC-C69A27DE5CEB";
          RTCObj.PrjGuid = "29291891-7C5E-448E-B89C-D81BC0722BA9";
        }
        //
        // Aggiorno i componenti
        BeginCompRequest();
        //
        // Inizio richiesta (analisi UserAgent)
        BeginRequest();
        //
        // Verifico se non  una richiesta a WebAPI
        WebApiObject.IsWebApiRequest = (WebApiObject.CheckUri().Length != 0);
        //
        // Solo se non  un item di test... il test chiama l'IWApp al momento di partire e
        // la init verr fatta in quel momento
        if ((req != null && (req["WCI"] == null || !req["WCI"].Equals(MyGlb.ITEM_TEST))) || IsServerSession())
          RolObj.InitRoles(); // Scatena la InitApp!
        //
        if (MyParamsObj().ResponseBufferSize > 8)
        {
//          Response.setBufferSize(MyParamsObj().ResponseBufferSize * 1024);
        }
        //
        if (Request != null)
        {
          WebItem = req["WCI"];
          WebEvent = req["WCE"];
          UrlData = req["WCU"];
        }
        if (WebItem == null)
          WebItem = "";
        if (WebEvent == null)
          WebEvent = "";
        if (UrlData == null)
          UrlData = "";
        //
        SetUrlData(UrlData);
        //
        if (Request != null)
        {
          if (!WebItem.Equals(MyGlb.ITEM_BLOB) && !WebItem.Equals(MyGlb.ITEM_STREAM))
          {
            Response.BufferOutput = true;
            //
            // Uso UTF8 solo se sono in RD3 (in RD2 devo usare Latin1 per il frame RD)
            bool useUTF8 = false;
            if (ParamsObj().UseRD3)
              useUTF8 = true;
            //
            // Anche per la sincronizzazione rispondo sempre in UTF-8
            if (WebItem.Equals(MyGlb.ITEM_SYNC))
              useUTF8 = true;
            //
            OpenResponse(false);
            Out = new StreamWriter(OutStr, useUTF8 ? Encoding.UTF8 : Encoding.Default);
            Response.ContentType = "text/html";
          }
          else
            OpenResponse(true); // No compressione per i blob
        }
        //
        // Ri-aggiorno i componenti... potrebbe essere cambiato qualcosa (es: nati ruoli, creato stream Out)...
        BeginCompRequest();
        //
        // Se c' il comando URL, chiamo l'evento
        //
        IDVariant cmd = GetUrlParam();
        if (cmd.stringValue().Length > 0)
        {
          DTTObj.SetRequestName("Command " + cmd.ToString(), "Query String Command Detected", DTTRequest.PRI_DATA);
          //
          bool ro = RolObj.CheckRole();
          bool cr = false;
          if (ro)
            cr = CmdObj.ExecCmdCode(cmd.stringValue()); // Solo se utente gi loggato!
          if (!ro || !cr)
          {
            // Lancio evento, non era gestita da un comando
            DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 152, "MainFrm.BeginRequest", "Fire OnCommand(" + cmd.ToString() + ") event");
            OnCommand(cmd);
            if (!ro && RolObj.CheckRole())
            {
              MainFrm.DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 114, "OnCommand", "Bypass login (Session Role set while handling a command)");
              // No login will occur, need to activate app manually
              //
              FireAfterLogin();
            }
            //
            // Se  il CMD che indica il ritorno online di una app OWA
            if (RolObj.CheckRole() && cmd.stringValue().Equals("OWABackOnline"))
            {
              WebItem = Glb.ITEM_RD3;
              WebEvent = "OWABackOnline";
            }
          }
          //
          if (cmd.stringValue().Equals("?IDVER"))
            DumpVersions(true);
          if (cmd.stringValue().Equals("?DEBUG"))
            OpenDebugWindow();
          if (cmd.stringValue().Equals("?DTTOK"))
          {
            DTTObj.DTTStarted = true;
            set_AlertMessage(new IDVariant("DTT Started"));
          }
          if (cmd.stringValue().Equals("?DTTKO"))
          {
            DTTObj.DTTStarted = false;
            set_AlertMessage(new IDVariant("DTT Stopped"));
          }
          //
          // a CMD is arrived, disable Session Number checking
          IntSesNumber = 0;
        }
        //
        // Dopo NPQ 940 (migliorato LOGIN1.HTM passando WCI, WCE e WCU in POST) c' un problema
        // se premo F5... se mi arriva IWLOGIN ed ho gi un ruolo attivo, annullo l'item
        if (WebItem.Equals(MyGlb.ITEM_LOGIN) && RolObj.CheckRole())
        {
          WebItem = "";
          WebEvent = "";
          UrlData = "";
          //
          // Se c' l'RD3 meglio resettare il tutto
          if (ParamsObj().UseRD3)
            RD3Reset();
        }
        //
        if (!IsServerSession())
        {
          // Selezione del WEBITEM tramite stringa
          //
          try
          {
            if (WebItem.Equals(MyGlb.ITEM_APP))
              IWApp_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_LOGIN))
              IWLogin_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_LOGOFF))
              IWLogoff_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_MENU))
              IWMenu_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_SBIND))
              IWSbInd_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_FORM))
              IWForm_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_FORMLIST))
              IWFormList_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_BLOB))
              IWBlob_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_UPLOAD))
              IWUpload_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_FILES))
              IWFiles_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_TREE_EXP))
              IWTreeExp_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_TREE_CLICK))
              IWTreeClick_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_TREE_DROP))
              IWTreeDrop_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_GRAPH_CLICK))
              IWGraphClick_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_DTT))
              IWDTT_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_TEST))
              IWTest_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_HELP))
              IWHelp_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_FRAMES))
              IWFrames_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_RD))
              IWRD_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_W3C))
              IW3C_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_RD3))
              IWRD3_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_SYNC))
              IWSync_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_STREAM))
              IWStream_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_OWA))
              IWOwa_UserEvent(WebEvent);
            else if (WebItem.Equals(MyGlb.ITEM_OAUTH))
              IWOAuth_UserEvent(WebEvent);
            else if (!WebApiObject.HandleRequest())
            {
              if (UseRD)
              {
                if (!DTTObj.TestEnabled && !iParamsObj.UseRD3)
                  IWFrames_UserEvent(WebEvent);
                else
                  IWApp_UserEvent(WebEvent);
              }
              else
                IWApp_UserEvent(WebEvent);
            }
          }
          catch (Exception ex)
          {
            MainFrm.DTTObj.AddException("", "MainFrm", "Exception while handling event WCI=" + WebItem + " WCE=" + WebEvent + " URL=" + UrlData, ex);
            MainFrm.ErrObj.ProcError("MainFrm", "WCI=" + WebItem + " WCE=" + WebEvent + " URL=" + UrlData, ex);
            RenderPage();
          }
        }
      }
      catch (Exception e)
      {
        FatalErrorResponse(e);
      }
      finally
      {
        try
        {
          EndRequest();
        }
        catch (Exception e1)
        {
          Console.Error.WriteLine(e1.Message + "\n" + e1.StackTrace); Console.Error.Flush();
        }
        //
        try
        {
          if (Out != null)
            Out.Close();
          //
          if (OutStr != null)
            OutStr.Close();
        }
        catch (Exception e)
        {
          Console.Error.WriteLine(e.Message + "\n" + e.StackTrace); Console.Error.Flush();
        }
        //
        Request = null;
        Response = null;
        OutStr = null;
        Out = null;
        //
        // Chiudo i files di log se li ho aperti
        if (OldOutStream != null)
        {
          Console.Out.Close();
          Console.SetOut(OldOutStream);
        }
        if (OldErrStream != null)
        {
          Console.Error.Close();
          Console.SetError(OldErrStream);
        }
        //
        // Componenti
        EndCompRequest();
      }
    }
  }

  public override void FireAfterLogin()
  {
    RolObj.ApplyRoles(0, null);
    DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 20, "MainFrm", "Fire AfterLogin event");
    AfterLogin();
    TimerObj.ActivateTimers(0, true);
  }

  // **********************************************
  // Check for user login
  // **********************************************
  public override void IWLogin_UserEvent(String EventName)
  {
    // Siccome posso arrivare qui dall'RD3 se l'utente preme F5, devo
    // mostrare nuovamente il desktop
    RD3Reset();
    //
    // Check for user data...
    IDVariant Valid, sUser, sPass;
    //
    sUser = new IDVariant(Glb.HTMLDecode(Request["UserName"]));
    sPass = new IDVariant(Glb.HTMLDecode(Request["PassWord"]));
    Valid = new IDVariant(true);
    RolObj.glbUser = new IDVariant(sUser);
    //
    DTTObj.SetRequestName("Login", "Login procedure", DTTRequest.PRI_ITEM);
    DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 150, "MainFrm.Login", "Fire OnLogin event");
    DTTObj.AddTestItem("USERNAME", DTTEngine.TST_APPLOGUP, sUser.stringValue(), "");
    if (sUser.stringValue().Length == 0)
      DTTObj.AddHelpItem(0, 0, 0, 0, "Lascia vuoto il campo UserName");
    else
      DTTObj.AddHelpItem(0, 0, 0, 0, "Scrivi " + sUser.stringValue() + " nel campo UserName");
    DTTObj.AddTestItem("PASSWORD", DTTEngine.TST_APPLOGUP, sPass.stringValue(), "");
    if (sPass.stringValue().Length == 0)
      DTTObj.AddHelpItem(0, 0, 0, 0, "Lascia vuoto il campo Password");
    else
      DTTObj.AddHelpItem(0, 0, 0, 0, "Scrivi " + sPass.stringValue() + " nel campo Password");
    DTTObj.AddTestItem("LOGIN", DTTEngine.TST_APPLOGIN, "", "");
    DTTObj.AddHelpItem(0, 0, 0, 0, "Clicca sul bottone LOGIN");
    MainFrm.Login(sUser, sPass, Valid);
    //
    if (RolObj.CheckRole())
      FireAfterLogin();
    else
      DTTObj.AddMsg(DTTEngine.DTTMSG_WARN, "", 8, "MainFrm.Login", "Login failed (User Role not set)");
    //
    RenderPage();
  }


  // **********************************************
  // Premuto pulsante logoff
  // **********************************************
  public override void IWLogoff_UserEvent(String EventName)
  {
    String s = "";
    //
    DTTObj.SetRequestName("Logoff", "Logoff button pressed", DTTRequest.PRI_CMD);
    DTTObj.AddTestItem("", DTTEngine.TST_LOGOFF, "Logoff", "");
    DTTObj.AddHelpItem(0, 0, 0, 0, "Clicca sul bottone LOGOFF");    
    //
    // Lancio Evento
    IDVariant skip = new IDVariant();
    IDVariant cancel = new IDVariant();
    DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 197, "MainFrm.OnLogoff", "Logoff button pressed: firing OnLogoff Event");      
    OnLogoff(skip, cancel);
    if (cancel.isTrue())
    {
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 197, "MainFrm.OnLogoff", "Logoff was canceled by OnLogoff Event");
      if (RDStatus == MyGlb.RD_ACTIVE)
        RenderDifferences();
      return;
    }
    if (skip.isFalse())
    {
      // Tento di chiudere tutte le form
      ArrayList c = new ArrayList(StackForm);
      IEnumerator i = c.GetEnumerator();
      //    
      while (i.MoveNext())
      {
        WebForm f = (WebForm) i.Current;
        UnloadForm(f.FormIdx, false);
      }
      //
      if (DockedForm()!=null)
        UnloadForm(DockedForm().FormIdx, false);
      //
      if (StackForm.Count>0)
        cancel = IDVariant.TRUE;
    }
    if (cancel.isTrue())
    {
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 197, "MainFrm.Logoff", "Logoff was canceled because not all forms can be closed");
      if (RDStatus == MyGlb.RD_ACTIVE)
        RenderDifferences();
      return;
    }
    //
    if (DTTObj.TestRunning || DTTObj.RecordRunning)
    {
      Out.Write("<html>");
      Out.Write("<head>");
      Out.Write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">");
      Out.Write("<script language=\"javascript\" type=\"text/javascript\">");
      if (DTTObj.RecordRunning)
        Out.Write("top.window.close();");
      else
        Out.Write("window.parent.frames('Control').NextStep();");
      Out.Write("</script>");
      Out.Write("</head>");
      Out.Write("</html>");
      //
      // Mettiamo sempre e solo un messaggio (se no ci saranno delle differenze!)
      //if (DTTObj.RecordRunning)
      //  DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 23, "MainFrm.Logoff", "Record termination");
      //else
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 23, "MainFrm.Logoff", "Test termination");
    }
    else
    {
      if (LogoffURL.Length > 0 && !LogoffURL.Equals("???"))
        s = LogoffURL;
      else
      {
        if (UseRD)
        {
          if (ParamsObj().UseRD3)
            s = AppName;
          else if (DTTObj.TestEnabled)
            s = UrlFor(MyGlb.ITEM_APP, "");
          else
            s = UrlFor(MyGlb.ITEM_FRAMES, "");
        }
        else
          s = UrlFor(MyGlb.ITEM_APP, "");
      }
      //
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 23, "MainFrm.Logoff", "Redirect to " + s);
      if (RDStatus == MyGlb.RD_ACTIVE)
      {
        RDExitURL = s;
        RenderDifferences();
      }
      else
      {
        try
        {
          if (ParamsObj().UseRD3)
            RD3AddEvent("redirect",s);
          else
            Response.Redirect(s, false);
        }
        catch (Exception e)
        {
          Console.Error.WriteLine(e.Message + "\n" + e.StackTrace); Console.Error.Flush();
        }
      }
    }
    //
    HttpContext.Current.Session.Abandon();
  }


  // **********************************************
  // Fatal Error (a component or framework error)
  // **********************************************
  public override void FatalErrorResponse(Exception e)
  {
    base.FatalErrorResponse(e);
    //
    if (Out != null)
    {
      Out.Write(e.StackTrace); Out.Flush();
    }
    //
    Console.Error.WriteLine(e.Message + "\n" + e.StackTrace); Console.Error.Flush();
  }


  // **********************************************
  // Before end, close database connections
  // **********************************************
  public override void EndRequest()
  {
    //
    //
    //
    base.EndRequest();
  }


  // **********************************************
  // Opens the right output stream
  // **********************************************
  public override void OpenResponse(bool flNoCompr)
  {
    try
    {
      if (!flNoCompr && MyParamsObj().CompressResponse.Equals("YES"))
      {
        String encodings = Request.Headers["Accept-Encoding"];
        if (encodings != null && encodings.IndexOf("gzip") != -1)
        {
          // GZIP
          if (encodings.IndexOf("x-gzip") != -1)
            Response.AddHeader("Content-Encoding", "x-gzip");
          else
            Response.AddHeader("Content-Encoding", "gzip");
          OutStr = new GZipStream(Response.OutputStream, CompressionMode.Compress);
        }
        else
        {
/*
          if (encodings != null && encodings.IndexOf("compress") != -1)
          {
            // ZIP
            if (encodings.IndexOf("x-compress") != -1)
              Response.AddHeader("Content-Encoding", "x-compress");
            else
              Response.AddHeader("Content-Encoding", "compress");
            //
            ZipOutputStream z = new ZipOutputStream(Response.OutputStream);
            z.putNextEntry(new ZipEntry("dummy name"));
            OutStr = z;
          }
          else
*/
          {
            // No compression
            OutStr = Response.OutputStream;
          }
        }
        Response.AddHeader("Vary", "Accept-Encoding");
      }
      else
      {
        // Compression is disabled
        OutStr = Response.OutputStream;
      }
    }
    catch (Exception)
    {

    }
  }


  // **********************************************
  // Render current page
  // **********************************************
  public override void RenderPage()
  {
    // Se  attivo l'RD4, vado di ShowDestop... non ho altro da aggiungere
    if (ParamsObj().UseRD4)
    {
      ShowDesktop();
      return;
    }
    //
    // In RD3 avviato, RD2 avviato o RD0 se non  gi stato programmato un messaggio all'utente vedo se  previsto un riavvio dell'applicazione
    if (((UseRD && iParamsObj.UseRD3 && DesktopRendered) || (UseRD && !iParamsObj.UseRD3 && RDStatus == Glb.RD_ACTIVE) || !UseRD) && AlertMessage().stringValue().Length == 0)
      CheckRestartWarning();
    //
    // Se il debug  attivo (non TRACE), vedo se occorre mostrare errori all'utente
    if (DTTObj.Level > 0 && !DTTObj.TraceEnabled && !ErrObj.IsError())
      DTTObj.ShowReqErrors();
    //
    // Popup menu
    int PopupMenu_CmdSetIdx = 0;
    int PopupMenu_FormIdx = 0;
    //
    if (DisableRender)
    {
      DisableRender = false;
      return;
    }
    //
    bool onlyform = WidgetMode;
    //
    String s = Request["WCU"];
    int p = (s == null) ? -1 : s.IndexOf("TT");
    if (p > -1)
    {
      p = p + 2;
      int f = p;
      while (f < s.Length && Char.IsDigit(s[f]))
        f++;
      try
      {
        TreeScrollTop = Int32.Parse(s.Substring(p, f - p));
      }
      catch (Exception e)
      {
        TreeScrollTop = 0;
      }
    }
    //
    set_ClickCounter(ClickCounter() + 1);
    RefreshNextClick = false;
    //
    SetUrlData("");
    //
    // True Popup
    if (RDStatus!=Glb.RD_DISABLED && ParamsObj().TruePopup)
    {
      if (IEVer >= 600 && ActiveForm() != null)
        onlyform = (ActiveForm().OpenAs != Glb.OPEN_MDI) || WidgetMode;
    }
    //
    // Need to redirect?
    //
    if ((!iParamsObj.UseRD3 || Request["WCI"]=="IWBlob") && RDStatus != MyGlb.RD_ACTIVE && RedirectTo().stringValue().Length > 0 && !RedirectNewWindow() && !DTTObj.TestRunning)
    {
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 6, "MainFrm.RenderPage", "Redirecting to " + RedirectTo().ToString());
      //
      // Se sono in RD3 (e la richiesta  di tipo IWBlob: vedi sopra)
      // e se devo gestire i file in streaming ...
      if (iParamsObj.UseRD3 && DeleteAfterDownload)
      {
        // Questo  l'unico caso in RD3 nel quale serve inviare dell'HTML
        // in quanto il download di un blob apre gi una nuova finestra e non si aspetta dell'XML
        // Codice replicato dall'RD3RenderDifferences
        try
        {
          // Se il file  locale all'applicazione
          FileInfo f = new FileInfo(RealPath + Path.DirectorySeparatorChar + RedirectTo().stringValue());
          if (f.Exists)
          {
            set_RedirectTo(new IDVariant(UrlFor(Glb.ITEM_STREAM)));
            FileToSend = f.FullName;
          }
        }
        catch (Exception)
        {
        }
      }
      //
      WriteStr("<html>\n");
      WriteStr("<head>\n");
      WriteStr("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">\n");
      WriteStr("<title>\n");
      WriteStr("" + GetPageTitle() + "\n");
      WriteStr("</title>\n");
      WriteStr("<link rel=stylesheet type=\"text/css\" href=\"iw.css\">\n");
      WriteStr("</head>\n");
      WriteStr("<frameset>\n");
      WriteStr("<frame src=\"" + RedirectTo() + "\">\n");
      WriteStr("</frameset>\n");
      WriteStr("</html>\n");
      //
      // Se c' RD3, lo spedisce e svuota lui... a meno che non sia uno scaricamento di BLOB
      // In questo caso occorre svuotarlo comunque!
      if (!iParamsObj.UseRD3 || Request["WCI"]=="IWBlob")
        set_RedirectTo(new IDVariant());
      //
      //RefreshNextClick = True;
      return;
    }
    //
    // Updating IMDB
    //
    TimerObj.Tick(this);
    UpdateIMDBState();
    //
    if (!RolObj.CheckRole())
    {
      // Show login screen
      //
      ShowLogin();
      return;
    }
    //
    ResBuf.Length = 0;
    //
    // Gestione del Trace
    DTTTrace();
    //
    // Gestione rendering differenziale
    if (UseRD)
    {
      if (RDStatus == MyGlb.RD_ACTIVE)
      {
        RenderDifferences(onlyform);
        return;
      }
      //
      if (iParamsObj.UseRD3 && !PreDesktopRendered)
      {
        PreDesktopRendered = true;
        //
        WriteStr("<html><head>");
        WriteStr("<script>window.location.href = window.location.href;</script>");
        WriteStr("</head></html>");
        return;
      }
      //
      if(iParamsObj.UseRD3 && DesktopRendered)
      {
        RenderDifferences3();
        return;
      }
      //
      if(iParamsObj.UseRD3 && !DesktopRendered)
      {
        ShowDesktop();
        DesktopRendered = true;
        return;
      }
    }
    //
    if (ParamsObj().HtmlConstraints == MyGlb.WEBVALID_STRICT)
      WriteStr("<!DOCTYPE html PUBLIC \"-//W3C//DTD HTML 4.01//EN\" \"http://www.w3.org/TR/html4/strict.dtd\">");
    WriteStr("<html>");
    WriteStr("<head>");
    WriteStr("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">");
    WriteStr("<meta http-equiv=\"X-UA-Compatible\" content=\"IE=EmulateIE7\">");
    if (ParamsObj().IWCacheControl.Equals("CACHE") || UseRD)
    {
      WriteStr("<meta http-equiv=\"Pragma\" content=\"no-cache\">");
      WriteStr("<meta http-equiv=\"Expires\" content=\"-1\">");
    }
    if (!UseRD)
    {
      if (RefreshInterval().intValue() > -1)
        WriteStr("<meta http-equiv=\"refresh\" content=\"" + RefreshInterval().intValue() + ";url=" + UrlFor(MyGlb.ITEM_APP, "") + "\">");
    }
    WriteStr("<title>");
    WriteStr(GetPageTitle());
    WriteStr("</title>");
    WriteStr("<link rel=stylesheet type=\"text/css\" href=\"iw.css\">");
    if (DTTObj.HelpGenerationActive)
      WriteStr("<link rel=stylesheet type=\"text/css\" href=\"dtt.css\">");
    WriteStr("<link rel=stylesheet type=\"text/css\" href=\"custom.css\">");
    //
    // client mask?
    if (ParamsObj().UseClientMask)
      WriteStr("<script type=\"text/javascript\" src=\"maskedinp.js\"></script>");
    WriteStr("<script type=\"text/javascript\" src=\"ijlib.js\"></script>");
    WriteStr("<script type=\"text/javascript\" src=\"dragdrop.js\"></script>");
    WriteStr("<script type=\"text/javascript\" src=\"custom.js\"></script>");
    //
    // Form JScript events
    //
    WriteStr("<script type=\"text/javascript\">");
    WriteStr("function InitBody() { ");
    if (!ParamsObj().UseFK)
      WriteStr("UseFK=false;");
    else
    {
      WriteStr("HLColorED=\"" + Glb.GetWebColor(VisualStyleList.Color(1, Glb.VISCLR_EDITING)) + "\";");
      if (VisualStyleList.GradCol(1, Glb.VISCLR_EDITING) != -1)
        WriteStr("HLColorEDFlt=\"" + Glb.GetWebGradient(VisualStyleList.Color(1, Glb.VISCLR_EDITING), VisualStyleList.GradCol(1, Glb.VISCLR_EDITING), VisualStyleList.GradDir(1, Glb.VISCLR_EDITING)) + "\";");
      else
        WriteStr("HLColorEDFlt=\"" + Glb.GetWebOpacity(VisualStyleList.Opacity(1, Glb.VISCLR_EDITING)) + "\";");
    }
    if (!ParamsObj().UseDoubleClick)
      WriteStr("UseDBLCLK=false;");
    if (ParamsObj().HtmlConstraints == MyGlb.WEBVALID_STRICT)
      WriteStr("StrictHTML=true;");
    if (ParamsObj().UseClientMask)
    {
      WriteStr("glbDecSep='" + (ParamsObj().UseDecimalDot ? '.' : ',') + "';");
      WriteStr("glbThoSep='" + (ParamsObj().UseDecimalDot ? ',' : '.') + "';");
    }
    WriteStr(" ResizeTables();");
    WriteStr(" ChangeRowDelay=" + (ParamsObj().UseFK?ParamsObj().ChangeRowDelay:0) + ";");
    if (ActiveForm() != null)
    {
      WriteStr("try {");
      if (ActiveForm().ScrollTop >= 0)
        WriteStr(" document.body.scrollTop = " + ActiveForm().ScrollTop + "; ");
      if (ActiveForm().ActiveElement.Length > 0)
        WriteStr(" if (document.all(\"" + ActiveForm().ActiveElement + "\")!=null) document.all(\"" + ActiveForm().ActiveElement + "\").focus(); ");
      WriteStr("} catch (e) {}");
    }
    if (TreeScrollTop > 0)
    {
      WriteStr("try {");
      WriteStr("document.getElementById(\"TREE\").scrollTop = " + TreeScrollTop + ";");
      WriteStr("} catch (e) {}");
    }
    if (RedirectNewWindow() && !DTTObj.TestRunning)
    {
      set_RedirectTo(IDL.Replace(RedirectTo(), new IDVariant("\\"), new IDVariant("/")));
      if (RedirectFeatures().stringValue().ToLower().IndexOf("modal") > -1)
        WriteStr("window.showModalDialog(" + MyGlb.JSEncode2(RedirectTo()));
      else
        WriteStr("window.open(" + MyGlb.JSEncode2(RedirectTo()));
      if (RedirectFeatures().stringValue().Length > 0)
        WriteStr(", null, " + MyGlb.JSEncode(RedirectFeatures()));
      WriteStr("); ");
    }
    if (AlertMessage()!=null && AlertMessage().stringValue().Length > 0 && !DTTObj.TestRunning && !DTTObj.HelpGenerationActive)
      WriteStr("alert(" + MyGlb.JSEncode2(AlertMessage()) + "); ");
    //
    // Stampo senza RD?
    RenderPDFPrints();
    //
    WriteStr(" } </script>");
    //
    WriteStr("</head>");
    WriteStr("<body");
    if (WidgetMode)
      WriteStr(" style=\"overflow:hidden\"");
    else if (ParamsObj().HtmlConstraints != MyGlb.WEBVALID_NONE)
      WriteStr(" style=\"overflow:visible\"");
    //
    String js = "InitBody();";
    //
    // Se il test  in esecuzione chiamo il prossimo step
    if (DTTObj.TestRunning || DTTObj.HelpGenerationActive)
      js = js + "try{window.parent.frames('Control').NextStep();}catch(e){}";
    //
    if (UseRD)
    {
      if (RDStatus != MyGlb.RD_DISABLED)
      {
        js = js + " ChangeTarget();";
        // Solo ora carico l'altro frame che far poi il change target
        js = js + " GetFrame(window.parent.frames,'RD').location.href='" + UrlFor(MyGlb.ITEM_RD, "") + "';";
      }
      if (UseRD && ParamsObj().TruePopup)
      {
        if (IEVer >= 600 && ActiveForm() != null)
        {
          WebForm af = ActiveForm();
          if (af.OpenAs != Glb.OPEN_MDI)
            js = js + " rp(" + af.FormLeft + ", " + af.FormTop + ", " + af.FormWidth + ", " + af.FormHeight + ");";
        }
      }
    }
    //
    // Se  stato richiesto il menu contestuale, lo mostro
    if (MenuIdx!="" && RDStatus!=Glb.RD_ACTIVE)
    {
      // Recupero la direzione corrente
      int Dir = 0;
      if (MenuIdx.IndexOf("|") != -1)
      {
        Dir = Int32.Parse(MenuIdx.Substring(MenuIdx.IndexOf("|") + 1));
        MenuIdx = MenuIdx.Substring(0, MenuIdx.IndexOf("|"));
      }
      else
        Dir = 1; // Default: Bottom
      //
      // Recupero il CmdSet ed il FormIdx
      String ObjIDs;
      int FormIdx;
      PopupMenu_CmdSetIdx = Int32.Parse(MenuIdx.Substring(0, MenuIdx.IndexOf(":")));
      MenuIdx = MenuIdx.Substring(MenuIdx.IndexOf(":") + 1);
      ObjIDs = MenuIdx.Substring(0, MenuIdx.IndexOf(":"));
      MenuIdx = MenuIdx.Substring(MenuIdx.IndexOf(":") + 1);
      PopupMenu_FormIdx = Int32.Parse(MenuIdx);
      if (PopupMenu_FormIdx == -1)
      {
        if (ActiveForm() != null)
          PopupMenu_FormIdx = ActiveForm().FormIdx;
        else
          PopupMenu_FormIdx = 0;
      }
      //
      // Scrivo il comando che posizioner il popup menu
      js = js + "popupshow_NoRD('" + ObjIDs + "|" + Dir + "');";
    }
    //
    // FK per custom commands...
    // NPQ 428: tasti FK anche con RD non attivo
    String FK = WebFrame.GetCCFK();
    if (FK.Length>0)
      js += "if (!StdFKInit) " + FK;
    //
    if (js != "")
      WriteStr(" onload=\"" + js + "\"");
    //
    set_RedirectNewWindow(false);
    set_RedirectTo(new IDVariant());
    set_RedirectFeatures(new IDVariant());
    set_AlertMessage(new IDVariant(""));
    WriteStr(">");
    //
    if (onlyform)
    {
      WriteStr("<table id=ExtTable cellspacing=\"0\" cellpadding=\"0\">"); // Primary Frame Definition
      WriteStr("<tr><td id=ActiveForm valign=\"top\">"); // Page Definition
      RenderActiveForm(true);
      WriteStr("</td></tr></table>");
    }
    else
    {
      WriteStr("<div id=hdr>");
      RenderHeader();
      WriteStr("</div><table id=ExtTable cellspacing=0 cellpadding=0>"); // Primary Frame Definition
      WriteStr("<tr>");
      //
      // Menu Bar
      WriteStr("<td id=MenuTable style=\"width:1px;vertical-align:top\">"); // Menu Frame Definition
      RenderMenuBar();
      //
      WriteStr("</td><td id=RightTables><table id=RightTable cellspacing=\"0\" cellpadding=\"0\"><tr><td id=StatusBarTable>");
      RenderStatusBar();
      WriteStr("</td></tr><tr><td id=ToolBarTable>");
      //
      if (CmdObj.IsToolbarPresent())
      {
        WriteStr("<table id=ToolBar><tr>"); // Tool Bar Definition
        //
        //
        CmdObj.RenderToolbar();
        //
        //
        WriteStr("</tr></table>");
      }
      WriteStr("</td></tr><tr>");
      //
      IntSesNumber = (int)(Glb.Random.NextDouble() * 2147483647); // New internal session number
      //
      WriteStr("<td id=PageTable>"); // Page Definition
      RenderPageTable();
      WriteStr("</td></tr>");
      //
      // Closing primary table
      WriteStr("</table></td></tr></table>");
    }
    //
    if ((ShowCalendar || RDStatus == Glb.RD_FRAMES) && (ParamsObj().HtmlConstraints != MyGlb.WEBVALID_STRICT || (UseRD && RDStatus != Glb.RD_DISABLED)))
    {
      // Creating popup calendar
      WriteStr("<script type=\"text/javascript\" src=\"calpopup.js\"></script>");
      WriteStr("<script event=onclick() for=document>");
      WriteStr("document.all.calpopup.style.display=\"none\";</script>");
      WriteStr("<iframe id=calpopup style=\"display:none;z-index:100;width:157px;height:162px;position:absolute\"");
      WriteStr("marginwidth=0 marginheight=0 src=\"calpopup.htm\" frameborder=0 noresize scrolling=no></iframe>");
      ShowCalendar = false;
    }
    //
    if (RDStatus == Glb.RD_FRAMES && UseRD)
      WriteStr("<iframe id=delaydlg style=\"display:none;z-index:100;position:absolute;filter:progid:DXImageTransform.Microsoft.Shadow(color=gray,direction=135,strength=3)\" marginwidth=5 marginheight=5 src=\"delaydlg.htm\" frameborder=0 noresize scrolling=no progrbar='" + ProgrBarFileName() + "'></iframe>");
    //    
    // Se  attivo l'Help generation aggiungo 2 iframes per le label di help
    if (DTTObj.HelpGeneration)
    {
      WriteStr("<div id=\"reqdiv\" class=HelpReqDiv style=\"display:none\"></div>");
      WriteStr("<div id=\"reqVdiv\" class=HelpReqDiv style=\"display:none\"></div>");
      WriteStr("<div id=\"itemdiv\" class=HelpItemDiv style=\"display:none\"></div>");
      WriteStr("<img id=\"helpcurs\" class=HelpCurs src=\"dttimg/helpcurs.gif\" style=\"display:none\">");
    }
    //
    // Se c'era da mostrare un menu popup, lo faccio
    if (MenuIdx!="" && RDStatus!=Glb.RD_ACTIVE)
    {
      CmdObj.RenderPopupMenu(PopupMenu_CmdSetIdx, PopupMenu_FormIdx);
      MenuIdx = "";
    }
    //
    // Inserisco l'oggetto per la stampa PDF
    WriteStr("<div id=\"dpdf\"></div>");
    RenderTrayletCommand(); // Renderizza l'IFRAME per chiamare la traylet!
    //
    WriteStr("</body>");
    //
    if ((ParamsObj().IWCacheControl.Equals("CACHE") || UseRD) && ParamsObj().HtmlConstraints != MyGlb.WEBVALID_STRICT)
    {
      WriteStr("<head>");
      WriteStr("<meta http-equiv=\"Pragma\" content=\"no-cache\">");
      WriteStr("<meta http-equiv=\"Expires\" content=\"-1\">");
      WriteStr("</head>");
    }
    WriteStr("</html>");
    SetUrlData(""); // Disabilito URL per evitare problemi al debugger
    //
    if (UseRD)
    {
      if (RDStatus != MyGlb.RD_DISABLED)
        ResetRD();
    }
  }


  // **********************************************
  // Esegue render docked e active form
  // **********************************************
  public override void RenderPageTable()
  {
    //
    WriteStr("\n");
    WriteStr("\n");
    //
    WriteStr("<table class=wh100 cellspacing=\"0\" cellpadding=\"0\"><tr>");
    LastDockType = (DockedForm() != null) ? DockedForm().DockType : 0;
    if (LastDockType == 0 || LastDockType == Glb.FORMDOCK_LEFT || LastDockType == Glb.FORMDOCK_TOP)
    {
      WriteStr("<td id=dockform style=\"height:10px\">");
      RenderDockedForm();
    }
    else
    {
      WriteStr("<td id=ActiveForm>");
      RenderActiveForm();
    }
    //
    WriteStr("</td>");
    if (LastDockType == Glb.FORMDOCK_TOP || LastDockType == Glb.FORMDOCK_BOTTOM)
      WriteStr("</tr><tr>");
    //
    if (LastDockType == 0 || LastDockType == Glb.FORMDOCK_LEFT || LastDockType == Glb.FORMDOCK_TOP)
    {
      WriteStr("<td id=ActiveForm>");
      RenderActiveForm();
    }
    else
    {
      WriteStr("<td id=dockform style=\"height:10px\">");
      RenderDockedForm();
    }
    WriteStr("</td></tr></table>");
    //
    WriteStr("\n");
    WriteStr("\n");
    //
  }


  // **********************************************
  // Status bar clicking events
  // **********************************************
  public override void IWSbInd_UserEvent(String EventName)
  {
    if (RefreshNextClick && !ParamsObj().UseRD3)
    {
      RenderPage();
      return;
    }
    //
    DispatchToComp("IWSbInd_UserEvent", new Object[] { EventName });
    //
    if (RolObj.CheckRole())
    {
      //
      //
      // Cerco l'indicatore corrispondente
      AIndicator ind = null;
      for (int i = 1; i < IndObj.Indicators.Length; i++)
      {
        if (IndObj.Indicators[i] == null)
          continue;
        //
        if (IndObj.Indicators[i].Code.Equals(EventName))
        {
          ind = IndObj.Indicators[i];
          break;
        }
      }
      if (ind != null)
      {
        DTTObj.SetRequestName("Indicator " + ind.Code + " click", "Indicator " + ind.Code + " click", DTTRequest.PRI_CMD);
        DTTObj.AddTestItem(ind.iGuid, DTTEngine.TST_INDIC, "Indicator " + ind.Code + " action", "");
        DTTObj.AddHelpItem(0, 0, 0, 0, "Premi l'indicatore " + ind.Code);
      }
    }
    else
      DTTObj.AddMsg(DTTEngine.DTTMSG_WARN, "", 19, "MainFrm.StatusBarHandler", "Unhandled Status Bar command: undefined session role");
    //
    if (!ParamsObj().UseRD3)
      RenderPage();
  }

  // **********************************************
  // W3C Validation
  // **********************************************
  public override void IW3C_UserEvent(String EventName)
  {
    // Salvo il vecchio stato
    int oldrd = RDStatus;
    bool oldhlp = DTTObj.HelpGenerationActive;
    //
    // Disabilito l'RD
    RDStatus = Glb.RD_DISABLED;
    //
    // Direct Output (vedi WriteStr)
    DTTObj.HelpGenerationActive = false;
    //
    WriteStr("<html>\n");
    WriteStr("<head>\n");
    WriteStr("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">\n");
    WriteStr("<title>Validating HTML</title>\n");
    WriteStr("<link rel=stylesheet type=\"text/css\" href=\"iw.css\">\n");
    WriteStr("<script type=\"text/javascript\">\n");
    WriteStr("function done()\n");
    WriteStr("{\n");
    WriteStr("theform.submit();\n");
    WriteStr("setTimeout('n()', 400);\n");
    WriteStr("}\n");
    WriteStr("function n()\n");
    WriteStr("{\n");
    WriteStr("prog.innerText += '.';\n");
    WriteStr("setTimeout('n()', 400);\n");
    WriteStr("}\n");
    WriteStr("</script>\n");
    WriteStr("</head>\n");
    WriteStr("<body onload=\"done()\" style=\"background-color:white\">\n");
    WriteStr("<center>\n");
    WriteStr("<table style=\"margin-top:30px;border-top: 2 dotted #000080;border-bottom: 2 dotted #000080\" cellpadding=\"5\" cellspacing=\"5\">\n");
    WriteStr("<tr><td><img src=\"dttimg/idi.gif\"></td><td style=\"font-size:26pt;font-weight:bold;color:gray\">Validating HTML</td></tr>\n");
    WriteStr("<tr><td>&nbsp;</td><td id=prog style=\"font-size:26pt;font-weight:bold;color:gray\"></td></tr></table></center>\n");
    WriteStr("<form id=\"theform\" method=\"post\" enctype=\"multipart/form-data\" action=\"http://validator.w3.org/check\">\n");
    WriteStr("<input id=\"ss\" name=\"ss\" type=hidden value=\"1\"/>\n");
    WriteStr("<textarea style=\"display:none\" cols=\"10\" rows=\"10\" name=\"fragment\" id=\"fragment\">\n");
    //
    // Buffered Output (vedi WriteStr)
    DTTObj.HelpGenerationActive = true;
    //
    // Renderizzo la pagina encodandola
    ResBuf.Length = 0;
    RenderPage();
    Out.Write(Glb.HTMLEncode(ResBuf.ToString()));
    ResBuf.Length = 0;
    //
    // Direct Output (vedi WriteStr)
    DTTObj.HelpGenerationActive = false;
    //
    WriteStr("</textarea>\n");
    WriteStr("</form>\n");
    WriteStr("</body>\n");
    WriteStr("</html>\n");
    //
    // Ripristino il tutto
    RDStatus = oldrd;
    DTTObj.HelpGenerationActive = oldhlp;
  }

  // **********************************************
  // Gestione delle FORMS
  // **********************************************
  public override void Show(int FormIdx, int flModal)
  {
    Show(FormIdx, flModal, null);
  }
  public override void Show(int FormIdx, int OpenAs, Object ParentObj)
  {
    int Owner = 0;
    IDVariant Cancel = new IDVariant();
    //
    // Se non  RD3 allora la form deve essere parte dell'MDI
    if (OpenAs == Glb.OPEN_POPUP && RDLevel() < 3)
    {
      OpenAs = Glb.OPEN_MDI;
    }
    //
    // Check if docked form is open
    if (DockedForm() != null && DockedForm().FormIdx == FormIdx)
    {
      DockedForm().set_Visible(true);
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 14, "MainFrm.Show", "Form " + DockedForm().Caption() + " already open and docked");
      return;
    }
    //
    // Check is form is already open
    WebForm f = GetFormByIndex(FormIdx);
    if (f != null)
    {
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 14, "MainFrm.Show", "Form " + f.Caption() + " already open: bringing to front");
      f.set_Visible(true);
      //
      // In RD2 non  mai docked perch  stato controllato prima, in RD3 pu invece essere docked: in quel caso saltiamo la BringToFront e 
      // scriviamo il messaggio che avremmo scritto in RD2
      if (f.Docked)
      {
        DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 14, "MainFrm.Show", "Form " + f.Caption() + " already open and docked");
        //
        if (UseZones() && ScreenZone(f.DockType) != null)
        {
          ScreenZone(f.DockType).set_SelectedForm(f.FormIdx);
        }
      }
      else
        BringToFront(FormIdx);
      return;
    }
    //
    RDForm = true; // Dovr ridisegnare l'intera form!
    WebForm OldForm = ActiveForm();
    //
    // Se non  MDI o sono su mobile (dove memorizzo l'owner anche se  MDI)
    WebForm parentF = null;
    if (OpenAs != Glb.OPEN_MDI || IsMobile())
    {
      // Se la videata  MDI non tengo conto dell'ActiveForm
      parentF = (OpenAs != Glb.OPEN_MDI ? ActiveForm() : null);
      if (ParentObj != null && ParentObj is WebForm)
        parentF = (WebForm)ParentObj;
      //
      // Se da una DOCKED apro una MDI, non le attacco tra loro
      // Ma se  SmartPhone devo comunque attaccarle perch su smartphone c' una sola videata aperta
      if (parentF != null && parentF.Docked && OpenAs == Glb.OPEN_MDI && !IsSmartPhone())
        parentF = null;
      //
      if (parentF != null)
      {
        Owner = parentF.FormIdx;
        //
        if (OpenAs != Glb.OPEN_MDI)
        {
          // Dato che la nuova form  modale, controllo se la form corrente vuole deattivarsi
          // Se non vuole... non apro la nuova form
          parentF.Form_Deactivate(Cancel);
          parentF.SubForm_Deactivate(Cancel);
          if (Cancel.isTrue())
          {
            DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 12, "MainFrm.Show", "The new form has not been shown, because the parent form " + parentF.Caption() + " refuses to deactivate (cancel := true)");
            return;
          }
        }
      }
      else if (OpenAs != Glb.OPEN_MDI)
        Owner = 32767; // Altrimenti non viene riconosciuta come form modale
    }
    //
    WebForm newform = null;
  fine:
    if (newform != null)
      set_ActiveForm(newform);
    //
    ActiveForm().ActivedTick = ClickCounter() + 1;
    StackForm.Insert(0, ActiveForm());
    ActiveForm().Owner = Owner;
    //
    // Inizializzo il testo del back-button
    if (parentF != null && parentF.OpenAs == Glb.OPEN_MDI && OpenAs == Glb.OPEN_MDI && !ActiveForm().Docked)
      ActiveForm().set_BackButTxt(parentF.Caption().stringValue());
    //
    if (ParamsObj().TruePopup && UseRD)
    {
      if (OpenAs != 0 && IEVer >= 600)
      {
        // Se avevo gi programmato la chiusura della stessa form, la annullo... la vecchia
        // form verr sostituita da questa. Altrimenti programmo l'apertura.
        // Sempre che la modale non sia stata chiusa in malomodo... in quel caso... devo
        // aprirne una nuova!
        if (RDClosePopup && !RDPopupClosedForcibly && FormIdx==ClosingForm)
          RDClosePopup = false;
        else
          RDOpenPopup = true;
      }
      //
      // Se c'era gi una chiusura in corso di una popup e la nuova form
      // che  stata aperta non  una popup, aspetto a chiudere la modale
      if (RDClosePopup)
        RDDelayPopup = true;
    }
    //
    ActiveForm().OpenAs = OpenAs;
    //
    WebForm oldActiveForm = ActiveForm();
    ActiveForm().Init(this, false, false);
    //
    // Se il programmatore non ha gi definito il testo del back button da s:
    // su smartphone mostro il back-button anche nel caso di modali con closeonselection 
    // (per poterle chiudere senza selezione) solo se la videata non mostra la sua caption 
    // ma quella del pannello (se lo fa i bottoni X e V sono gi presenti nella toolbar della videata)
    if (ActiveForm() != null && ActiveForm().BackButTxt().Length == 0 && parentF != null && OpenAs == Glb.OPEN_MODAL && IsSmartPhone() && ActiveForm().CloseOnSelection && ActiveForm().VisualFlag(Glb.FORMVISPROP_HASCAPTION) == 0)
      ActiveForm().set_BackButTxt(parentF.Caption().stringValue());
    //
    // Se  cambiata la form attiva pu essere successo che  stata chiusa la videata appena aperta...
    if (oldActiveForm != ActiveForm())
    {
      // Se la vecchia form, quella appena aperta,  ancora nello stack form... devo proseguire
      bool flIsStillOpen = false;
      IEnumerator i = StackForm.GetEnumerator();
      while (i.MoveNext())
      {
        if ((WebForm)i.Current == oldActiveForm)
        {
          flIsStillOpen = true;
          break;
        }
      }
      //
      // Se  stata chiusa... non devo proseguire!
      if (!flIsStillOpen)
        return;
    }
    //
    // Proseguo... informo l'RD3 che deve aprire questa form
    RD3Event ev = RD3AddEvent("open", "frm:" + FormIdx);
    if (OpenAs != 0)
      ev.SetAttribute("openas", OpenAs.ToString());
    //
    // Se  cambiata l'ActiveForm, allora potrebbe essere successo che nell'evento di OnLoad:
    // 1)  stata aperta o attivata un'altra form
    // 2) la form  stata resa invisibile
    // 3) la form  stata chiusa
    // In questo caso attivo la vecchia form, se non ce ne  nessuna attiva
    if (oldActiveForm != ActiveForm())
    {
      // Se  uno dei casi 2 o 3 e non c'erano altre form gi aperte, ActiveForm diventa NULL
      // In questo caso ripristino la form attiva
      if (ActiveForm() == null)
        set_ActiveForm(OldForm);
      //
      // Informo l'RD3 che occorre attivare questa form
      if (ActiveForm() != null)
        RD3AddEvent("activateform", "frm:" + ActiveForm().FormIdx);
      return;
    }
    //
    // Docked Forms
    if (ActiveForm().Docked)
    {
      if (!ParamsObj().UseRD3)
      {
        RDDockForm = true;
        if (DockedForm() != null) // Sostituzione Docked Form
        {
          UnloadForm(DockedForm().FormIdx, false);
          set_DockedForm(null); // Forzo Chiusura
        }
        StackForm.Remove(ActiveForm());
        set_DockedForm(ActiveForm());
        set_ActiveForm(OldForm);
        //
        f = DockedForm();
        f.Form_Activate(); // Fire activate event
        f.SubForm_Activate();
        //
        // Ridimensiono il client size!
        if (f.DockType == Glb.FORMDOCK_LEFT || f.DockType == Glb.FORMDOCK_RIGHT)
          ResizeClient(ScreenW() - f.OrgWidth + f.GetPadding(false) + 1, ScreenH(), false);
        else
          ResizeClient(ScreenW(), ScreenH() - f.OrgHeight + f.GetPadding(true) + 1, false);
        return;
      }
      else
      {
        // Gestione Form Docked RD3
        // Devo sostituire un'eventuale DockedForm sullo stesso lato
        if (!UseZones())
        {
          foreach (WebForm frm in StackForm)
          {
            if (frm.Docked && frm.DockType == ActiveForm().DockType && frm.FormIdx != ActiveForm().FormIdx)
            {
              UnloadForm(frm.FormIdx, false);
              break;
            }
          }
        }
        //
        WebForm dkf = ActiveForm();
        set_ActiveForm(OldForm);
        //
        if (UseZones() && ScreenZone(dkf.DockType) != null)
          ScreenZone(dkf.DockType).set_SelectedForm(dkf.FormIdx);
        else
        {
          dkf.Form_Activate(); // Fire activate event
          dkf.SubForm_Activate();
        }
        //
        return;
      }
    }
    
    //
    // Se  modale ho gi chiesto
    if (OldForm != null && OpenAs == 0)
    {
      OldForm.Form_Deactivate(Cancel);
      OldForm.SubForm_Deactivate(Cancel);
    }
    //
    if (Cancel.isFalse())
    {
      SetHelpFile(ActiveForm().HelpFile);
      f = ActiveForm();
      f.Form_Activate(); // Fire activate event
      f.SubForm_Activate();
    }
    else
    {
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 12, "MainFrm.Show", "The form " + ActiveForm().Caption() + " has been shown, but the form " + OldForm.Caption() + " refuses to deactivate (cancel := true)");
      set_ActiveForm(OldForm);
      //
      RD3AddEvent("activateform", "frm:" + ActiveForm().FormIdx);
    }
  }

  // **********************************************
  // Crea una nuova istanza di form a partire dall'indice
  // **********************************************
  public override WebForm CreateFormByIndex(int FormIdx)
  {
    WebForm newform = null;
  fine:
    if (newform != null)
    {
      newform.MainFrm = this;
      return newform;
    }
    //
    // Se non ci sono riuscito, provo con i miei componenti
    foreach (WebEntryPoint cmp in CompList)
    {
      newform = cmp.CreateFormByIndex(FormIdx);
      if (newform != null)
        return newform;
    }
    //
    return null;
  }

  // **********************************************
  // Gestione delle FORMS / (by name)
  // **********************************************
  public override bool Show(String FormName, int OpenAs)
  {
    return Show(FormName, OpenAs, null);
  }
  public override bool Show(String FormName, int OpenAs, Object parentObj)
  {
    bool ris = false;
    //
    try
    {
      WebForm newform = (WebForm)System.Reflection.Assembly.GetExecutingAssembly().CreateInstance(FormName, true);
      newform.Init(this, false, false);
      Show(newform.FormIdx, OpenAs, parentObj);
    }
    catch(Exception e)
    {
      ;
    }
    return ris;
  }


  // **********************************************************************
  // Open database connection as needed
  // **********************************************************************
  public override void OpenConnection(IDConnection DB)
  {
    if (DB == null || DB.IsOpen())
      return; // Not a connection or not closed
    //
  }


  // **********************************************
  // Command Activation Handler
  // **********************************************
  public override void CmdClickCB(int CmdIdx)
  {
    DispatchToComp("CmdClickCB", new Object[] { CmdIdx });
    if (CmdIdx==MyGlb.CMD_PROCEDURE1+BaseCmdLinIdx)
    {
      MainFrm.Procedure1();
      goto fine;
    }
    fine: ;
  }


  // **********************************************
  // Timer Activation Handler
  // **********************************************
  public override void TimerTickCB(int TimerIdx)
  {
    DispatchToComp("TimerTickCB", new Object[] { TimerIdx });
      fine: ;
  }


  // **********************************************
  // Get HTML for logoff Button
  // **********************************************
  public override String LogoffButton()
  {
    String HF = "";
    String DTT = "";
    String TEL = "";
    String s = "";

    if (ActiveHelpFile().Length > 0)
      HF = "<a onclick=\"return OpenDoc('" + ActiveHelpFile() + "','Help','" + s + "');\" href=\"\" id=help><img alt=\"Open Help File\" src=\"images/help.gif\" style=\"border:none; background-image:url(images/btn_bk.gif);\"></a>&nbsp;";
    //
    if (DTTObj.Level > 0 && !DTTObj.TraceEnabled)
      DTT = "<a onclick=\"return OpenDoc('" + UrlForEnc(MyGlb.ITEM_DTT, "") + "','Debug','');\" href=\"\"><img alt=\"Open Debug Window\" src=\"images/bug.gif\" style=\"border:none; background-image:url(images/btn_bk.gif);\"></a>&nbsp;";
    //
    if (DTTObj.HelpEnabled && DTTObj.CanSendHelpMail(false))
      TEL = "<a href=\"" + UrlFor(MyGlb.ITEM_APP, "") + "&amp;CMD=DTTHELP\"><img alt=\"Request help\" src=\"images/dtthelp.gif\" style=\"border:none; background-image:url(images/btn_bk.gif);\"></a>&nbsp;";
    //
    if (LogoffURL.Length > 0)
      return HF + TEL + DTT + "<a id=\"LOGOFF\" href=\"" + UrlForEnc(MyGlb.ITEM_LOGOFF, "") + "\"><img alt=\"" + RTCObj.GetConst("38ABB688-409C-43A8-BA9F-1ECACD08D957", "Chiude l'applicazione", true) + "\" src=\"images/closex.gif\" style=\"border:none; background-image:url(images/btn_bk.gif);\"></a>";
    else
      return HF + TEL + DTT;
  }

  public String DebugButton()
  {
    if (DTTObj.Level > 0 && !DTTObj.TraceEnabled)
      return "<a href=\"" + UrlForEnc(MyGlb.ITEM_DTT, "") + "\" target=Debug><img alt=\"Open Debug Window\" src=\"images/bug.gif\" style=\"border:none; background-image:url(images/btn_bk.gif);\"></a>";
    //
    return "";
  }

  // **********************************************
  // Show login page
  // **********************************************
  public override void ShowLogin()
  {
    // Per il problema dell'F5 dovr rispondere con una paginetta di pre-desktop
    PreDesktopRendered = false;
    //
    WriteStr("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">\n");
    WriteStr("<html>\n");
    WriteStr("<head>\n");
    WriteStr("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">\n");
    WriteStr("<meta http-equiv=\"X-UA-Compatible\" content=\"IE=EmulateIE7\" />\n");
    WriteStr("<link rel=stylesheet type=\"text/css\" href=\"dtt.css\">\n");
    WriteStr("<title>Login Page</title>\n");
    WriteStr("<script language=\"javascript\" type=\"text/javascript\">\n");
    WriteStr("function PlayTest() { try { window.parent.frames('Control').NextRequest(); } catch(e) {}; }\n");
    WriteStr("</script>\n");
    WriteStr("</head>\n");
    WriteStr("\n");
    WriteStr("<body style=\"margin:0;\" onload=\"PlayTest()\">\n");
    WriteStr("<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"border-collapse: collapse; background-color: transparent;\" bordercolor=\"#111111\" width=\"100%\" height=\"100%\">\n");
    WriteStr("<tr>\n");
    WriteStr("<td colspan=3 style=\"background-color:#dee7f6\">&nbsp;</td>\n");
    WriteStr("</tr>\n");
    WriteStr("<tr>\n");
    WriteStr("<td style=\"background-color:#dee7f6;\">&nbsp;</td>\n");
    WriteStr("<td width=\"700\" height=\"450\" valign=\"top\" style=\"background-color: #dee7f6;\">\n");
    WriteStr("<table cellspacing=0 cellpadding=0 style=\"background-color: transparent\">\n");
    WriteStr("<tr>\n");
    WriteStr("<td style=\"width: 8px; height: 8px; background-image:url('images/wtl.gif');\"></td>\n");
    WriteStr("<td style=\"height: 8px; background-image:url('images/wtop.gif');\"></td>\n");
    WriteStr("<td style=\"width: 8px; height: 8px; background-image:url(images/wtr.gif);\"></td>\n");
    WriteStr("</tr>\n");
    WriteStr("<tr>\n");
    WriteStr("<td style=\"width: 8px; background-image:url(images/wleft.gif);\"></td>\n");
    WriteStr("<td style=\"height: 434px; width: 684px; background-color: white; background-image:url(images/splash.gif); background-repeat:no-repeat; background-position: bottom left;\">\n");
    WriteStr("\n");
    WriteStr("             <div style=\"position:relative;top:-30px;left:665px; width: 30px\">" + DebugButton() + "</div>\n");
    WriteStr("<p align=\"center\">\n");
    WriteStr("<img border=\"0\" src=\"images/pow2.gif\" width=\"590\" height=\"70\"><p align=\"center\">\n");
    WriteStr("&nbsp;\n");
    WriteStr("<table border=\"0\" cellpadding=\"0\" cellspacing=\"0\" style=\"border-collapse: collapse\" bordercolor=\"#111111\" width=\"100%\" height=\"20\">\n");
    WriteStr("<tr>\n");
    WriteStr("<td width=\"30%\" height=\"20\">&nbsp;</td>\n");
    WriteStr("<td width=\"67%\" height=\"20\"><b><font face=\"Trebuchet MS\" size=\"4\">\n");
    WriteStr("<form method=\"post\" autocomplete=\"off\" name=\"frm\" action=\"HandlingDebug.aspx\">\n");
    WriteStr("<input type=hidden name=WCI value=IWLogin>\n");
    WriteStr("<input type=hidden name=WCE value=Form1>\n");
    WriteStr("<input type=hidden name=WCU>\n");
    WriteStr("                 <font color=\"#BCBC77\">" + Glb.HTMLEncode(Caption(), true, true) + " - </font>\n");
    WriteStr("<font color=\"#004d89\">Login Form</font></b>\n");
    WriteStr("<p><font color=\"#004d89\" face=\"Trebuchet MS\" size=\"2\">Username&nbsp;&nbsp;\n");
    WriteStr("                 <input id=\"USERNAME\" name=\"UserName\" tabindex=\"1\" size=\"20\" style=\"border: 1px solid #BCBC77;color:#004d89;font-family:Trebuchet MS;width:150px;height:20px\" value=\"" + Glb.HTMLEncode(RolObj.glbUser) + "\" />\n");
    WriteStr("<br>Password&nbsp;&nbsp;&nbsp;\n");
    WriteStr("<input id=\"PASSWORD\" type=\"password\" name=\"PassWord\" tabindex=\"2\" size=\"20\" style=\"border:1px solid #BCBC77;color:#004d89;width:150px;height:20px\" >\n");
    WriteStr("<br>&nbsp;</p>\n");
    WriteStr("<p><input id=\"LOGIN\" border=\"0\" src=\"images/login.gif\" name=\"I1\" width=\"83\" height=\"28\" type=\"image\"></p>\n");
    WriteStr("                 <p><font face=\"Trebuchet MS\" size=\"2\"><font color=\"#004d89\">" + MainFrm.RTCObj.GetConst("4D691B94-E528-4AEC-B319-0F18A9049DB3", "Introduci il tuo Username e&nbsp;Password per accedere al sistema, oppure<br>clicca qui per tornare alla <a href=\"http://www.progamma.com\">pagina iniziale</a>", false) + "</font></font></p>\n");
    WriteStr("               	<p style=\"padding-right:40px\"><b><font color=\"#C04040\" face=\"Trebuchet MS\" size=\"2\">" + Glb.HTMLEncode(LoginMessage, true, true) + "</font></b></p>\n");
    WriteStr("<SCRIPT>\n");
    WriteStr("if (document.frm.UserName.value.length==0)\n");
    WriteStr("          					document.frm.UserName.focus();\n");
    WriteStr("else\n");
    WriteStr("          					document.frm.PassWord.focus();\n");
    WriteStr("</SCRIPT>\n");
    WriteStr("</form></td>\n");
    WriteStr("</tr>\n");
    WriteStr("</table>\n");
    WriteStr("\n");
    WriteStr("</td>\n");
    WriteStr("<td style=\"width: 8px; background-image:url(images/wright.gif);\"></td>\n");
    WriteStr("</tr>\n");
    WriteStr("<tr>\n");
    WriteStr("<td style=\"width: 8px; height: 8px; background-image:url(images/wbl.gif);\"></td>\n");
    WriteStr("<td style=\"height: 8px; background-image:url(images/wbottom.gif);\"></td>\n");
    WriteStr("<td style=\"width: 8px; height: 8px; background-image:url(images/wbr.gif);\"></td>\n");
    WriteStr("</tr>\n");
    WriteStr("</table>\n");
    WriteStr("\n");
    WriteStr("</td>\n");
    WriteStr("<td style=\"background-color:#dee7f6\">&nbsp;</td>\n");
    WriteStr("</tr>\n");
    WriteStr("<tr>\n");
    WriteStr("<td style=\"background-color:#dee7f6\" colspan=3>&nbsp;</td>\n");
    WriteStr("</tr>\n");
    WriteStr("</table>\n");
    WriteStr("<div id=\"reqdiv\" class=HelpReqDiv style=\"display:none\"></div>\n");
    WriteStr("<div id=\"reqVdiv\" class=HelpReqDiv style=\"display:none\"></div>\n");
    WriteStr("<div id=\"itemdiv\" class=HelpItemDiv style=\"display:none\"></div>\n");
    WriteStr("<img id=\"helpcurs\" class=HelpCurs src=\"dttimg/helpcurs.gif\" style=\"display:none\">\n");
    WriteStr("</body>\n");
    WriteStr("</html>\n");
    //
  }


  // **********************************************
  // Imposta una url data
  // **********************************************
  public override void SetUrlData(String UD)
  {
    URLData = "";
    if (ParamsObj().IWCacheControl.Equals("URL"))
      URLData = ((double)(Glb.Random.NextDouble() * 2147483647)).ToString();
    //    
    URLData = URLData + UD;
  }

  // **********************************************
  // Esegue il rendering differenziale...
  // **********************************************
  public override void RenderDifferences(bool onlyform)
  {
    String ObjToFocus = "";
    if (ActiveForm() != null)
      ObjToFocus = ActiveForm().ActiveElement;
    //
    // Cappello iniziale
    Out.Write("<html>");
    Out.Write("<head>");
    Out.Write("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=windows-1252\">");
    //
    // La disabilitazione della cache del frame RD NON pu essere effettuata mentre
    // apro una popup, altrimenti IE impazzisce e ne apre tantissime.
    if (!RDOpenPopup)
    {
      Out.Write("<meta http-equiv=\"Expires\" content=\"-1\">");
      Out.Write("<meta http-equiv=\"Pragma\" content=\"no-cache\">");
    }
    //
    Out.Write("<script language=\"javascript\" src=\"ijlib.js\"></script>");
    Out.Write("<script language=\"javascript\" src=\"dragdrop.js\"></script>");
    Out.Write("<script language=\"javascript\" src=\"custom.js\"></script>");
    Out.Write("<script language=\"javascript\" type=\"text/javascript\">");
    Out.Write("function ApplyRender() { ");
    //
    // Invio nuovamente i dati aggiornati (potrebbero essere cambiati a run-time!)
    if (ParamsObj().UseClientMask)
      Out.Write("  SetDD('" + (UseDecimalDot() ? '.' : ',') + "','" + (UseDecimalDot() ? ',' : '.') + "');");
    //
    if (!RDOpenPopup && !RDClosePopup)
    {
      if (!onlyform)
      {
        int dt = (DockedForm() != null) ? DockedForm().DockType : 0;
        if (dt!=LastDockType)
        {
          // Devo fare il render dell'intera DOCK+FORM area
          ResBuf.Length = 0;
          RenderPageTable();
          SendRenderCmd("PageTable");
          RDForm = false;
          RDDockForm = false;
        }
        //
        if (RDHeader)
        {
          // Devo fare il rendering dell'intera header bar
          ResBuf.Length = 0;
          RenderHeader();
          SendRenderCmd("hdr");
        }
        //
        if (CmdObj.RDAll)
        {
          // Devo fare il rendering dell'intera men bar
          ResBuf.Length = 0;
          RenderMenuBar();
          SendRenderCmd("MenuTable");
        }
        else
        {
          // Analizzo interno del men
          CmdObj.RenderMenuDifference();
        }
      }
      //
      if (RDForm || (ActiveForm() != null && ActiveForm().RDUpload))
      {
        // Devo fare il rendering dell'intera active form
        // Resetto i tasti funzione che qui possono essere diversi...
        Out.Write(WebFrame.GetCCFK());
        ResBuf.Length = 0;
        RenderActiveForm();
        SendRenderCmd("ActiveForm");
        ResBuf.Length = 0;
        RenderFormListInternal();
        SendRenderCmd("FormList");
      }
      else
      {
        if (ActiveForm() != null)
        {
          if (ActiveForm().RDCaption && !CmdObj.SuppressMenu())
          {
            ResBuf.Length = 0;
            RenderFormListInternal();
            SendRenderCmd("FormList");
          }
          ActiveForm().RenderDifference();
        }
      }
      //
      if (!onlyform)
      {
        if (RDDockForm)
        {
          // Devo fare il rendering dell'intera active form
          ResBuf.Length = 0;
          RenderDockedForm();
          SendRenderCmd("DockForm");
        }
        else
        {
          if (DockedForm() != null)
            DockedForm().RenderDifference();
        }
        //
        if (RDSB || RDForm)
        {
          ResBuf.Length = 0;
          RenderStatusBar();
          SendRenderCmd("StatusBarTable");
        }
        //
        if (CmdObj.IsTBRD())
        {
          ResBuf.Length = 0;
          if (CmdObj.IsToolbarPresent())
          {
            WriteStr("<table id=ToolBar><tr>"); // Tool Bar Definition            
            //
            //
            CmdObj.RenderToolbar();
            //
            //
            WriteStr("</tr></table>");
          }
          SendRenderCmd("ToolBarTable");
        }
      }
    }
    else
    {
      if (RDOpenPopup && RDForm && ActiveForm() != null)
      {
        WebForm fo = GetForm(ActiveForm().Owner, 0, false);
        if (fo != null)
        {
          fo.RenderDifference();
          fo.ResetRD();
          Out.Write(" ApplyRD2();");
        }
      }
    }
    //
    // Cappello finale
    if (!RDOpenPopup && !RDClosePopup)
    {
      Out.Write(" window.setTimeout(\"ChangeTarget();\",10);");
      Out.Write(" ep('" + (ResTotal / 1024) + "-" + ResCount + (GZipEnabled == 0 ? " NOZIP" : "") + "');");
      Out.Write(" window.setTimeout(\"SetFocus('" + ObjToFocus + "')\",20);");
      if (!RedirectTo().stringValue().Equals("") && !DTTObj.TestRunning)
      {
        set_RedirectTo(IDL.Replace(RedirectTo(), new IDVariant("\\"), new IDVariant("/")));
        if (RedirectNewWindow())
        {
          bool ismodal = false;
          //
          if (RedirectFeatures().stringValue().ToLower().IndexOf("modal") != -1)
          {
            Out.Write("window.showModalDialog(" + MyGlb.JSEncode2(RedirectTo()));
            ismodal = true;
          }
          else
          {
            Out.Write("window.setTimeout(\"window.open(" + MyGlb.JSEncode2(RedirectTo(), "\\\""));
          }
          if (!RedirectFeatures().stringValue().Equals(""))
            Out.Write(", "+ (WidgetMode?"'browser'":"null")+ ", " + MyGlb.JSEncode(RedirectFeatures()));
          //
          if (ismodal)
            Out.Write("); ");
          else
            Out.Write(");\",200); ");
        }
        else
          Out.Write(" GetFrame(window.parent.frames,'Main').location.href=" + MyGlb.JSEncode2(RedirectTo()) + "; ");
        //
        // Gestita Redirect
        set_RedirectNewWindow(false);
        set_RedirectTo(new IDVariant(""));
        set_RedirectFeatures(new IDVariant(""));
      }
      //
      RenderPDFPrints();
    }
    //
    // Menu popup su PField
    if (!MenuIdx.Equals(""))
    {
      // Recupero la direzione corrente
      IDVariant Dir = null;
      if (MenuIdx.IndexOf("|") != -1)
      {
        Dir = new IDVariant(Int32.Parse(MenuIdx.Substring(MenuIdx.IndexOf("|") + 1)));
        MenuIdx = MenuIdx.Substring(0, MenuIdx.IndexOf("|"));
      }
      else
        Dir = new IDVariant(1); // Default: Bottom
      //
      // Recupero il CmdSet ed il FormIdx
      int CmdSetIdx;
      String ObjIDs;
      int FormIdx;
      CmdSetIdx = Int32.Parse(MenuIdx.Substring(0, MenuIdx.IndexOf(":")));
      MenuIdx = MenuIdx.Substring(MenuIdx.IndexOf(":") + 1);
      ObjIDs = MenuIdx.Substring(0, MenuIdx.IndexOf(":"));
      MenuIdx = MenuIdx.Substring(MenuIdx.IndexOf(":") + 1);
      FormIdx = Int32.Parse(MenuIdx);
      if (FormIdx == -1)
      {
        if (ActiveForm() != null)
          FormIdx = ActiveForm().FormIdx;
        else
          FormIdx = 0;
      }
      //
      // Mostro il menu
      ResBuf.Length = 0;
      CmdObj.RenderPopupMenu(CmdSetIdx, FormIdx);
      SendRenderCmd(ObjIDs + "|" + Dir.intValue(), "popup");
      MenuIdx = "";
    }
    //
    if (AlertMessage() != null && !AlertMessage().stringValue().Equals("") && !DTTObj.TestRunning && !DTTObj.HelpGenerationActive)
      Out.Write(" msg(" + MyGlb.JSEncode2(AlertMessage()) + "," + AlertType() + "); ");
    //
    if (RDOpenPopup && !RDDelayPopup)
    {
      Rect r = ActiveForm().Frames[1].GetTotalSize();
      int w = r.Width;
      int h = r.Height;
      String rectstr;
      //
      h = h + MyGlb.PAN_OFFS_PAGEY + MyGlb.PAN_OFFS_Y + 4 + 40; // Messages
      w = w + 8;
      if (w < 300)
        w = 300;
      if (h < 200)
        h = 200;
      //
      rectstr = "";
      if (ActiveForm().FormLeft != -1)
        rectstr = rectstr + "dialogLeft:" + ActiveForm().FormLeft + "px;";
      if (ActiveForm().FormTop != -1)
        rectstr = rectstr + "dialogTop:" + ActiveForm().FormTop + "px;";
      if (ActiveForm().FormWidth != -1)
        w = ActiveForm().FormWidth;
      if (ActiveForm().FormHeight != -1)
        h = ActiveForm().FormHeight;
      rectstr = rectstr + "dialogHeight:" + h + "px;";
      rectstr = rectstr + "dialogWidth:" + w + "px;";
      //
      Out.Write(" GetFrame(window.parent.frames,'Main').ModIdx=" + ActiveForm().FormIdx + ";");
      Out.Write(" window.showModalDialog(\"" + UrlFor(MyGlb.ITEM_FRAMES, "") + "\",window,\"" + rectstr + "help:no;status:no;resizable:yes;\");");
      Out.Write(" GetFrame(window.parent.frames,'Main').EnableClick=3;");
      Out.Write(" pb('WCI=IWRD&WCE=" + ActiveForm().FormIdx + "');");
      Out.Write(" window.defaultStatus=\"\";");
    }
    else
    {
      if (ScreenW() == 0 || ScreenH() == 0)
      {
        // Se c' una form aperta che si vuole ridimensionare faccio una richiesta
        bool ric = false;
        //
        if (DockedForm() != null && (DockedForm().ResModeH != Glb.FRESMODE_NONE || DockedForm().ResModeW != Glb.FRESMODE_NONE))
          ric = true;
        //
        IEnumerator i = StackForm.GetEnumerator();
        //    
        while (!ric && i.MoveNext())
        {
          WebForm f = (WebForm) i.Current;
          if (f.ResModeH != Glb.FRESMODE_NONE || f.ResModeW != Glb.FRESMODE_NONE)
            ric = true;
        }
        //
        if (ric)
          Out.Write(" window.setTimeout(\"pb('');\",10);");
      }
    }
    //
    if (RDClosePopup)
      Out.Write(" window.parent.close();");
    //
    if (!RDExitURL.Equals("") && RolObj.CheckRole())
    {
      Out.Write(" window.parent.location.href=" + MyGlb.JSEncode2(new IDVariant(RDExitURL)) + "; ");
      RDExitURL = "";
    }
    //
    if (RefreshInterval().intValue() > -1)
      Out.Write(" SetRefresh(" + (1000 * RefreshInterval().dblValue()) + "); ");
    //
    if (ActiveForm() != null && ActiveForm().ScrollTop >= 0)
      Out.Write(" sct(" + ActiveForm().ScrollTop + "); ");
    //
    Out.Write(" } </script></head><body onload=\"NextRD();\">");
    //
    // Form
    Out.Write("<form id=theform method=post action=\"" + UrlFor(MyGlb.ITEM_RD, "") + "\">");
    Out.Write("<input type=hidden id=ss name=ss>");
    Out.Write("<input type=hidden id=ace name=ace>");
    Out.Write("<input type=hidden id=pd name=pd>");
    Out.Write("<input type=hidden id=cmd name=cmd>");
    Out.Write("<input type=hidden id=keys name=keys>");
    RenderTrayletCommand(); // Renderizza l'IFRAME per chiamare la traylet!
    //
    // Vedi commenti sopra
    if (RDOpenPopup)
      Out.Write("</form></body></html>");
    else
      Out.Write("</form></body><head><meta http-equiv=\"Pragma\" content=\"no-cache\"><meta http-equiv=\"Expires\" content=\"-1\"></head></html>");
    //
    if (!RDOpenPopup && !RDClosePopup)
      ResetRD();
    if (RDDelayPopup)
      RDDelayPopup = false;
    else
      RDOpenPopup = false;
    RDClosePopup = false;
    if (AlertType() == 0)
    {
      ConfStack.Clear();
      set_AlertMessage(new IDVariant(""));
    }
  }

  // **********************************************
  // Esegue il rendering dell'header
  // **********************************************
  public void RenderHeader()
  {
    //
    WriteStr("<table border=0 style=\"width:100%\" cellspacing=0 cellpadding=0 id=ApplHeader>\n");
    WriteStr("<tr><td style=\"width:100%\">\n");
    WriteStr("" + MenuButton() + "&nbsp;" + Glb.HTMLEncode(Caption(), true, true) + "</td>\n");
    WriteStr("<td style=\"white-space:nowrap\"><span id=cmdspan class=cmdform>cmd: <input class=cmdinput type=text accesskey=\"c\" maxlength=6 name=cmd id=cmdcode size=6></span></td>\n");
    WriteStr("<td style=\"white-space:nowrap\">" + LogoffButton() + "</td>\n");
    WriteStr("</tr></table>\n");
    //
  }


  // **********************************************
  // Esegue il rendering della menu bar intera
  // **********************************************
  public void RenderMenuBar()
  {
    if (CmdObj.IsMenuPresent())
    {
      WriteStr("<table class=w100 id=Menu cellspacing=0 cellpadding=0>");
      //
      //
      CmdObj.RenderMenu();
      //
      //
      // Rendering window list
      //
      RenderFormList();
      //
      //
      WriteStr("</table>");
    }
  }

  // **********************************************
  // Esegue il rendering della menu bar intera
  // **********************************************
  public void RenderStatusBar()
  {
    bool Messages = false;
    //
    if (ActiveForm() != null)
      Messages = ActiveForm().HasMessages();
    if (IndObj.IsStatusBarPresent())
    {
      WriteStr("<div id=StatusBar>"); // Status Bar Definition
      //
      //
      IndObj.Render();
      //
      //
      WriteStr("</div>");
    }
    if (ActiveForm() != null)
      ActiveForm().RenderMessages();
  }

  // **********************************************
  // Esegue il rendering dell'intera form!
  // **********************************************
  public void RenderActiveForm() { RenderActiveForm(false); }
  public void RenderActiveForm(bool flmsg)
  {
    if (ErrObj.IsError())
    {
      ErrObj.Render();
      //RefreshNextClick = True
    }
    else
    {
      if (ActiveForm() != null)
      {
        ActiveForm().ActivedTick = ClickCounter();
        ActiveForm().Render(flmsg);
      }
      else
      {
        WriteStr("<iframe id=welcome src=\"" + WelcomeURL + "\" width=\"100%\" height=\"520px\" frameborder=\"0\"></iframe>\n");
      }
    }
  }

  // **********************************************
  // Get parameters
  // **********************************************
  public MyParams MyParamsObj()
  {
    return (MyParams)iParamsObj;
  }

  // **********************************************
  // Get DB by code
  // **********************************************
  public override IDConnection GetDBByName(String Name)
  {
    // Scateno l'evento... se vogliono mi cambiano il nome
    // dandomi il vero nome e preparando il DB come vogliono
    IDVariant TrueName = new IDVariant(Name);
    OnGetDBByName(TrueName);
    Name = TrueName.stringValue();
    //
    //
    if (CompOwner != null)
      return CompOwner.GetDBByName(Name);
    else
      return null;
  }

  public override void OnOpenPopup(ACommand SrcObj, IDVariant Direction, IDVariant Cancel)
  {
    DispatchToComp("OnOpenPopup", new Object[] { SrcObj, Direction, Cancel });
  }

  public override void OnCmdSetCommand(ACommand SrcObj, IDVariant CmdIdx, IDVariant ChildIdx, IDVariant Cancel)
  {
    DispatchToComp("OnCmdSetCommand", new Object[] { SrcObj, CmdIdx, ChildIdx, Cancel });
  }

  public override void OnCmdSetChangeExpand(ACommand SrcObj, IDVariant Expand, IDVariant Cancel)
  {
    DispatchToComp("OnCmdSetChangeExpand", new Object[] { SrcObj, Expand, Cancel });
  }
  
  public override void OnCmdSetGeneralDrag(ACommand SrcObj, IDVariant DragInfo, IDVariant CmdIdx, IDVariant ChildIdx)
  {
    DispatchToComp("OnCmdSetGeneralDrag", new Object[] { SrcObj, DragInfo, CmdIdx, ChildIdx });
  }
  
  public override void OnCmdSetGeneralDrop(ACommand SrcObj, IDVariant DragInfo, IDVariant CmdIdx, IDVariant ChildIdx)
  {
    DispatchToComp("OnCmdSetGeneralDrop", new Object[] { SrcObj, DragInfo, CmdIdx, ChildIdx });
  }

  // **********************************************
  // Update Controls against IMDB variations
  // **********************************************
  public override void UpdateControls()
  {
    DispatchToComp("UpdateControls", new Object[] { });
  }


  // **********************************************
  // Procedure Definition
  // **********************************************
  // **********************************************************************
  // Initialize
  // An event fired by the application when it is initialized
  // **********************************************************************
  public void InitApp ()
  {
    DispatchToComp("InitApp", new Object[] {});
    //
    StringBuilder SQL = new StringBuilder();
    int TransCount   = 0;
    int ReturnStatus = 0;
    IDCachedRowSet QV;

    try
    {
      TransCount = 0;

      if (!MainFrm.DTTObj.EnterProc("7C40583A-BBD3-4611-85AF-F7D9D2EEB76A", "Initialize", "An event fired by the application when it is initialized", 0, "MainFrm")) return;
      // 
      // Initialize Body
      // Procedure body
      // 
      MainFrm.DTTObj.AddAssign ("E4A6F67C-CFFF-4AD1-9147-08D8AFEAFCEF", "Handling Debug.User Role := Administrator", "", MainFrm.UserRole());
      MainFrm.DTTObj.AddToken ("E4A6F67C-CFFF-4AD1-9147-08D8AFEAFCEF", "85A3408B-A8F4-11D4-8F26-0860F2000000", 589824, "Administrator", (new IDVariant(1)));
      MainFrm.set_UserRole((new IDVariant(1)));
      MainFrm.DTTObj.AddAssignNewValue ("E4A6F67C-CFFF-4AD1-9147-08D8AFEAFCEF", "0DC03ED7-07F6-4CEA-9AAC-C69A27DE5CEB", MainFrm.UserRole());
      MainFrm.DTTObj.ExitProc("7C40583A-BBD3-4611-85AF-F7D9D2EEB76A", "Initialize", "An event fired by the application when it is initialized", 0, "MainFrm");
    }
    catch (Exception _e)
    {
      MainFrm.DTTObj.AddException("7C40583A-BBD3-4611-85AF-F7D9D2EEB76A", "Initialize", "An event fired by the application when it is initialized", _e);
      MainFrm.ErrObj.ProcError ("HandlingDebug", "Initialize", _e);
      MainFrm.DTTObj.ExitProc("7C40583A-BBD3-4611-85AF-F7D9D2EEB76A", "Initialize", "An event fired by the application when it is initialized", 0, "MainFrm");
    }
  }

  // **********************************************************************
  // On Login Event
  // An event fired by the application when the user performs
  // a login
  // Data Valid - Input/Output
  // User Name - Input/Output
  // Password - Input/Output
  // **********************************************************************
  public void Login (IDVariant UserName, IDVariant Password, IDVariant DataValid)
  {
    DispatchToComp("Login", new Object[] {UserName, Password, DataValid});
    //
    StringBuilder SQL = new StringBuilder();
    int TransCount   = 0;
    int ReturnStatus = 0;
    IDCachedRowSet QV;

    try
    {
      TransCount = 0;

      if (!MainFrm.DTTObj.EnterProc("371FC49C-FD42-4736-9360-59192D8ED945", "On Login Event", "An event fired by the application when the user performs a login", 0, "MainFrm")) return;
      MainFrm.DTTObj.AddParameter ("371FC49C-FD42-4736-9360-59192D8ED945", "0606F8CD-A794-4307-A8B7-8C79FC90F4AC", "Data Valid", DataValid);
      MainFrm.DTTObj.AddParameter ("371FC49C-FD42-4736-9360-59192D8ED945", "D2752C30-7402-4306-8333-F36C828AB4D7", "User Name", UserName);
      MainFrm.DTTObj.AddParameter ("371FC49C-FD42-4736-9360-59192D8ED945", "66F00A3C-59B5-412B-8BFA-99E67B4AF103", "Password", Password);
      // 
      // On Login Event Body
      // Procedure body
      // 
      MainFrm.DTTObj.AddAssign ("9EE04ACE-C997-4D23-90AF-4E820165DEB8", "Handling Debug.User Role := Administrator", "Administrator", MainFrm.UserRole());
      MainFrm.DTTObj.AddToken ("9EE04ACE-C997-4D23-90AF-4E820165DEB8", "85A3408B-A8F4-11D4-8F26-0860F2000000", 589824, "Administrator", (new IDVariant(1)));
      MainFrm.set_UserRole((new IDVariant(1)));
      MainFrm.DTTObj.AddAssignNewValue ("9EE04ACE-C997-4D23-90AF-4E820165DEB8", "0DC03ED7-07F6-4CEA-9AAC-C69A27DE5CEB", MainFrm.UserRole());
      MainFrm.DTTObj.AddAssign ("EC003171-7782-4783-AFE9-22F71F49A41F", "Data Valid := true", "true", DataValid);
      MainFrm.DTTObj.AddToken ("EC003171-7782-4783-AFE9-22F71F49A41F", "78B75DA3-DBDF-11D4-900E-726096000000", 589824, "true", (new IDVariant(-1)));
      DataValid.set((new IDVariant(-1)));
      MainFrm.DTTObj.AddAssignNewValue ("EC003171-7782-4783-AFE9-22F71F49A41F", "0606F8CD-A794-4307-A8B7-8C79FC90F4AC", DataValid);
      MainFrm.DTTObj.ExitProc("371FC49C-FD42-4736-9360-59192D8ED945", "On Login Event", "An event fired by the application when the user performs a login", 0, "MainFrm");
    }
    catch (Exception _e)
    {
      MainFrm.DTTObj.AddException("371FC49C-FD42-4736-9360-59192D8ED945", "On Login Event", "An event fired by the application when the user performs a login", _e);
      MainFrm.ErrObj.ProcError ("HandlingDebug", "OnLoginEvent", _e);
      MainFrm.DTTObj.ExitProc("371FC49C-FD42-4736-9360-59192D8ED945", "On Login Event", "An event fired by the application when the user performs a login", 0, "MainFrm");
    }
  }

  // **********************************************************************
  // After Login
  // An event fired by the application after the system
  // was accessed
  // **********************************************************************
  public void AfterLogin()
  {
    DispatchToComp("AfterLogin", new Object[] {});
    //
    StringBuilder SQL = new StringBuilder();
    int TransCount   = 0;
    int ReturnStatus = 0;
    IDCachedRowSet QV;

    try
    {
      TransCount = 0;

      if (!MainFrm.DTTObj.EnterProc("52035B57-9BC3-4EFD-BE70-3EFA05A6566F", "After Login", "An event fired by the application after the system was accessed", 0, "MainFrm")) return;
      // 
      // After Login Body
      // Procedure body
      // 
      MainFrm.DTTObj.AddAssign ("09340F68-CEED-4A35-8D67-D2A3E8A9692C", "Forms list.Expanded := true", "", MainFrm.CmdObj.CmdSetExpanded(MyGlb.CMDS_FORMSLIST+BaseCmdSetIdx));
      MainFrm.DTTObj.AddToken ("09340F68-CEED-4A35-8D67-D2A3E8A9692C", "78B75DA3-DBDF-11D4-900E-726096000000", 589824, "true", (new IDVariant(-1)).booleanValue());
      MainFrm.CmdObj.set_CmdSetExpanded(MyGlb.CMDS_FORMSLIST+BaseCmdSetIdx, (new IDVariant(-1)).booleanValue());
      MainFrm.DTTObj.AddAssignNewValue ("09340F68-CEED-4A35-8D67-D2A3E8A9692C", "06F2D873-573B-442A-A1B2-D2B1B5338E42", MainFrm.CmdObj.CmdSetExpanded(MyGlb.CMDS_FORMSLIST+BaseCmdSetIdx));
      MainFrm.DTTObj.ExitProc("52035B57-9BC3-4EFD-BE70-3EFA05A6566F", "After Login", "An event fired by the application after the system was accessed", 0, "MainFrm");
    }
    catch (Exception _e)
    {
      MainFrm.DTTObj.AddException("52035B57-9BC3-4EFD-BE70-3EFA05A6566F", "After Login", "An event fired by the application after the system was accessed", _e);
      MainFrm.ErrObj.ProcError ("HandlingDebug", "AfterLogin", _e);
      MainFrm.DTTObj.ExitProc("52035B57-9BC3-4EFD-BE70-3EFA05A6566F", "After Login", "An event fired by the application after the system was accessed", 0, "MainFrm");
    }
  }

  // **********************************************************************
  // Procedure 1
  // Describe what this procedure is used for
  // **********************************************************************
  public int Procedure1 ()
  {
    StringBuilder SQL = new StringBuilder();
    int TransCount   = 0;
    int ReturnStatus = 0;
    IDCachedRowSet QV;

    try
    {
      TransCount = 0;

      if (!MainFrm.DTTObj.EnterProc("D6123946-CBB5-4751-9622-6DF534DA25D0", "Procedure 1", "Describe what this procedure is used for", 3, "MainFrm")) return 0;
      // 
      // Procedure 1 body
      // Procedure body
      // 
      int DTT_C2 = 0;
      MainFrm.DTTObj.AddForI ("6EDB3D5A-C18D-42FD-BA7A-B793BAE8843E", "FOR (i := 0; i < 10; i := i + 1)", "Write a comment for this loop or press backspace to delete this comment");
      IDVariant I = new IDVariant(0,IDVariant.INTEGER);
      for (I = Procedure1_FI2_f1(I); Procedure1_FI2_f2(I); I = Procedure1_FI2_f3(I))
      {
        DTT_C2 = DTT_C2 + 1;
        if (!MainFrm.DTTObj.CheckLoop("6EDB3D5A-C18D-42FD-BA7A-B793BAE8843E", DTT_C2)) break;
        MainFrm.DTTObj.AddSubProc ("BEE72C70-EE7A-4073-958B-90A69DB419A0", "Handling Debug.Procedure 2", "Write a comment or press backspace to delete this comment");
        Procedure2();
      }
      MainFrm.DTTObj.EndForI ("6EDB3D5A-C18D-42FD-BA7A-B793BAE8843E", "FOR (i := 0; i < 10; i := i + 1)", "Write a comment for this loop or press backspace to delete this comment", DTT_C2);
      MainFrm.DTTObj.ExitProc("D6123946-CBB5-4751-9622-6DF534DA25D0", "Procedure 1", "Describe what this procedure is used for", 3, "MainFrm");
      return 0;
    }
    catch (Exception _e)
    {
      MainFrm.DTTObj.AddException("D6123946-CBB5-4751-9622-6DF534DA25D0", "Procedure 1", "Describe what this procedure is used for", _e);
      MainFrm.ErrObj.ProcError ("HandlingDebug", "Procedure1", _e);
      MainFrm.DTTObj.ExitProc("D6123946-CBB5-4751-9622-6DF534DA25D0", "Procedure 1", "Describe what this procedure is used for", 3, "MainFrm");
      return -1;
    }
  }

  private IDVariant Procedure1_FI2_f1(IDVariant I)
  {
    return I;
  }
  private bool Procedure1_FI2_f2(IDVariant I)
  {
    MainFrm.DTTObj.AddToken ("6EDB3D5A-C18D-42FD-BA7A-B793BAE8843E", "FB04D4E3-21A1-4BBD-A94D-033288EA13D5", 1376256, "i", I);
    return (I.compareTo((new IDVariant(10)), true)<0);
  }
  private IDVariant Procedure1_FI2_f3(IDVariant I)
  {
    MainFrm.DTTObj.AddAssign ("EB86CC1D-B807-4B25-B810-A8714D7CCA9C", "i := i + 1", "Write a comment or press backspace to delete this comment", I);
    MainFrm.DTTObj.AddToken ("EB86CC1D-B807-4B25-B810-A8714D7CCA9C", "FB04D4E3-21A1-4BBD-A94D-033288EA13D5", 1376256, "i", I);
    I = IDL.Add(I, (new IDVariant(1)));
    MainFrm.DTTObj.AddAssignNewValue ("EB86CC1D-B807-4B25-B810-A8714D7CCA9C", "FB04D4E3-21A1-4BBD-A94D-033288EA13D5", I);
    return I;
  }

  // **********************************************************************
  // Procedure 2
  // Describe what this procedure is used for
  // **********************************************************************
  public int Procedure2 ()
  {
    StringBuilder SQL = new StringBuilder();
    int TransCount   = 0;
    int ReturnStatus = 0;
    IDCachedRowSet QV;

    try
    {
      TransCount = 0;

      if (!MainFrm.DTTObj.EnterProc("213FB2A4-23D2-4B95-8EB5-E6FFC1BA9CDB", "Procedure 2", "Describe what this procedure is used for", 3, "MainFrm")) return 0;
      // 
      // Procedure 2 body
      // Procedure body
      // 
      MainFrm.DTTObj.LoggedLoops = (new IDVariant(1000)).intValue();
      int DTT_C2 = 0;
      MainFrm.DTTObj.AddForI ("6C92A8A7-388E-47E1-B473-1F9A5023CE72", "FOR (i := 0; i < 500; i := i + 1)", "Write a comment for this loop or press backspace to delete this comment");
      IDVariant I = new IDVariant(0,IDVariant.INTEGER);
      for (I = Procedure2_FI2_f1(I); Procedure2_FI2_f2(I); I = Procedure2_FI2_f3(I))
      {
        DTT_C2 = DTT_C2 + 1;
        if (!MainFrm.DTTObj.CheckLoop("6C92A8A7-388E-47E1-B473-1F9A5023CE72", DTT_C2)) break;
        IDVariant J = new IDVariant(0,IDVariant.INTEGER);
        MainFrm.DTTObj.AddAssign ("7C3CEB4B-DDA3-4092-A654-C924FCB820EC", "j := C(j + 5) * 2", "Write a comment or press backspace to delete this comment", J);
        MainFrm.DTTObj.AddToken ("7C3CEB4B-DDA3-4092-A654-C924FCB820EC", "265F4C23-DD07-4A72-BE06-8501FE9CDB9D", 1376256, "j", J);
        J = IDL.Mul((IDL.Add(J, (new IDVariant(5)))), (new IDVariant(2)));
        MainFrm.DTTObj.AddAssignNewValue ("7C3CEB4B-DDA3-4092-A654-C924FCB820EC", "265F4C23-DD07-4A72-BE06-8501FE9CDB9D", J);
      }
      MainFrm.DTTObj.EndForI ("6C92A8A7-388E-47E1-B473-1F9A5023CE72", "FOR (i := 0; i < 500; i := i + 1)", "Write a comment for this loop or press backspace to delete this comment", DTT_C2);
      MainFrm.DTTObj.ExitProc("213FB2A4-23D2-4B95-8EB5-E6FFC1BA9CDB", "Procedure 2", "Describe what this procedure is used for", 3, "MainFrm");
      return 0;
    }
    catch (Exception _e)
    {
      MainFrm.DTTObj.AddException("213FB2A4-23D2-4B95-8EB5-E6FFC1BA9CDB", "Procedure 2", "Describe what this procedure is used for", _e);
      MainFrm.ErrObj.ProcError ("HandlingDebug", "Procedure2", _e);
      MainFrm.DTTObj.ExitProc("213FB2A4-23D2-4B95-8EB5-E6FFC1BA9CDB", "Procedure 2", "Describe what this procedure is used for", 3, "MainFrm");
      return -1;
    }
  }

  private IDVariant Procedure2_FI2_f1(IDVariant I)
  {
    return I;
  }
  private bool Procedure2_FI2_f2(IDVariant I)
  {
    MainFrm.DTTObj.AddToken ("6C92A8A7-388E-47E1-B473-1F9A5023CE72", "CE06727C-3858-4207-8DF8-AC837BE9C6EE", 1376256, "i", I);
    return (I.compareTo((new IDVariant(500)), true)<0);
  }
  private IDVariant Procedure2_FI2_f3(IDVariant I)
  {
    MainFrm.DTTObj.AddAssign ("C1F46AE8-3C65-48FD-95E9-D4D34D14CF72", "i := i + 1", "Write a comment or press backspace to delete this comment", I);
    MainFrm.DTTObj.AddToken ("C1F46AE8-3C65-48FD-95E9-D4D34D14CF72", "CE06727C-3858-4207-8DF8-AC837BE9C6EE", 1376256, "i", I);
    I = IDL.Add(I, (new IDVariant(1)));
    MainFrm.DTTObj.AddAssignNewValue ("C1F46AE8-3C65-48FD-95E9-D4D34D14CF72", "CE06727C-3858-4207-8DF8-AC837BE9C6EE", I);
    return I;
  }



  // **********************************************
  // Event Stubs
  // **********************************************
  // **********************************************************************
  // On Logoff
  // An event fired by the application when the user clicks
  // on the application's logoff button
  // **********************************************************************
  public void OnLogoff(IDVariant Skip, IDVariant Cancel)
  {
    DispatchToComp("OnLogoff", new Object[] {Skip, Cancel});
    //
    // Stub
  }

  // **********************************************************************
  // Terminate
  // An event fired by the application immediately before
  // closing the user session
  // **********************************************************************
  public void TerminateApp ()
  {
    DispatchToComp("TerminateApp", new Object[] {});
    //
    // Stub
  }

  // **********************************************************************
  // On Resize
  // An event fired by the application when the browser
  // dimensions change
  // **********************************************************************
  public void OnResize()
  {
    DispatchToComp("OnResize", new Object[] {});
    //
    // Stub
  }

  // **********************************************************************
  // On Table Substitution
  // Event raised by the application when a query is executed
  // if its from list contains a table for which the "Table
  // substitution" flag is enabled
  // **********************************************************************
  public void OnTableSubstitution(IDVariant PreviousTableCode, IDVariant Join, IDVariant TableCode, IDVariant Reason)
  {
    DispatchToComp("OnTableSubstitution", new Object[] {PreviousTableCode, Join, TableCode, Reason});
    //
    // Stub
  }

  // **********************************************************************
  // Service Started
  // An event fired by the application when the service
  // is started
  // **********************************************************************
  private void NTStartService(IDVariant Success)
  {
    DispatchToComp("NTStartService", new Object[] {Success});
    //
    // Stub
  }

  // **********************************************************************
  // Service Stopped
  // An event fired by the application when the service
  // is stopped
  // **********************************************************************
  private void NTStopService()
  {
    DispatchToComp("NTStopService", new Object[] {});
    //
    // Stub
  }

  // **********************************************************************
  // Service Paused
  // An event fired by the application when the service
  // is paused
  // **********************************************************************
  private void NTPauseService(IDVariant Success)
  {
    DispatchToComp("NTPauseService", new Object[] {Success});
    //
    // Stub
  }

  // **********************************************************************
  // Service Continued
  // An event fired by the application when the service
  // is re-started
  // **********************************************************************
  private void NTContinueService(IDVariant Success)
  {
    DispatchToComp("NTContinueService", new Object[] {Success});
    //
    // Stub
  }



  // **********************************************
  // Show Desktop page
  // **********************************************
  public override void ShowDesktop()
  {
    if (RD3CompressFiles)
    {
      RenderAllImages(true);
    }
    else
    {
      // No compressione. Invio il file Desktop.htm
      StreamFile(RealPath + "/Desktop.htm");
    }
  }
  
  // **************************************************************
  // Gestisco l'arrivo di un comando RD3 interno all'XML
  // **************************************************************
  public override bool HandleCommand(XMLNode node)
  {
    // Leggo il nome del comando 
    String cmdvalue = node.GetAttribute("obn");
    // 
    // Inserisco il comando, e tutti gli eventuali attributi presenti, nella mappa dei parametri
    ExtractUrlParams2("CMD=" + cmdvalue);
    cmdvalue = GetUrlParam().stringValue();
    //
    // Replico la gestione del comando della MyWebEntryPoint (riga 327)
    DTTObj.SetRequestName("Command " + cmdvalue, "Query String Command Detected", DTTRequest.PRI_DATA);
    //
    bool ro = RolObj.CheckRole();
    bool cr = false;
    if (ro)
      cr = CmdObj.ExecCmdCode(cmdvalue); // Solo se utente gi loggato! (se arrivo qui dovrei essere sicuro, questo controllo non dovrebbe servire)
    if (!ro || !cr)
    {
      // Lancio evento, non era gestita da un comando
      DTTObj.AddMsg(DTTEngine.DTTMSG_INFO, "", 152, "MainFrm.BeginRequest", "Fire OnCommand(" + cmdvalue + ") event");
      OnCommand(new IDVariant(cmdvalue));
    }
    //
    if (cmdvalue.Equals("?IDVER"))
      DumpVersions(true);
    if (cmdvalue.Equals("?DEBUG"))
      OpenDebugWindow();
    if (cmdvalue.Equals("?DTTOK"))
    {
      DTTObj.DTTStarted = true;
      set_AlertMessage(new IDVariant("DTT Started"));
    }
    if (cmdvalue.Equals("?DTTKO"))
    {
      DTTObj.DTTStarted = false;
      set_AlertMessage(new IDVariant("DTT Stopped"));
    }
    //
    return true;
  }

  // **********************************************
  // Determina la lingua usata dall'applicazione
  // **********************************************
  public override String GetAppLanguage()
  {
    if (RTCObj.Enabled && RolObj.glbLanguage != ".")
      return RolObj.glbLanguage;
    else
      return MyParamsObj().LANGUAGE;
  }

  // **********************************************
  // Determina l'url dell'applicazione
  // **********************************************
  public override String GetAppUrl(bool defDoc, bool defPort)
  {
    // Solo l'applicazione sa rispondere correttamente
    if (CompOwner != null)
      return CompOwner.GetAppUrl(defDoc);
    //
    if (Request == null)
      return "";
    //
    String url = Request.Url.Scheme + "://";
    url += Request["SERVER_NAME"];
    if (Request.Url.Port > 0)
    {
      if (defPort && ((Request.Url.Scheme == "http" && Request.Url.Port != 80) || (Request.Url.Scheme == "https" && Request.Url.Port != 443)))
        url += ":" + Request.Url.Port;
    }
    url += Request.ApplicationPath;
    if (!url.EndsWith("/"))
      url += "/";
    //
    // Se si vuole il documento lo aggiungo
    if (defDoc)
      url += RD3EntryPoint;
    //
    return url;
  }

  // ************************************************************
  // Creo il file di manifest
  // ************************************************************
  public override void AddCustomToManifest(ArrayList CachedFiles) 
  {
    base.AddCustomToManifest(CachedFiles);
    //
  }

  // ************************************************************
  // Elenca i file utilizzati dall'applicazione offline
  // ************************************************************
  public override void GetFontsList(ArrayList Fonts)
  {
    base.GetFontsList(Fonts);
    //
  }
}
